/***************************************************************************
                          arc_zip.cpp  -  description
                             -------------------
    begin                : Sun Aug 20 2000
    copyright            : (C) 2000 by Morten Poulsen
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "arc_zip.h"
#include "sc_archive.h"

//////////////ZIP//////////////////////



/**************************************************/

// Routines for opening files inside archives

/**************************************************/



time_t MakeTimeFromUZT(const tm_unz& t) {

  //via tm-struct!  mktime in time.h!

  time_t result;

  tm via;

  via.tm_hour=t.tm_hour; via.tm_min=t.tm_min; via.tm_sec=t.tm_sec; via.tm_mon=t.tm_mon;

  via.tm_mday=t.tm_mday; via.tm_year=t.tm_year; //years since 1900

//#ifdef _TARGET_CYGWIN_

  via.tm_yday=via.tm_wday=via.tm_isdst=0;

//#else

//  via.tm_yday=via.tm_wday=via.tm_isdst=via.tm_gmtoff=0; via.tm_zone=NULL;

//#endif



  result=mktime(&via);

  return result;

}



bool ZIPScan(tDirContainer& cont, const sParsedDir& spd, bool onlydirs/*=false*/) {

  bool nodirrecs=true; //Bad form of zip-files

  unzFile uzp=unzOpen(spd.fullname.c_str());

  if (!uzp) return false;

  cont.Clear();

  if (!onlydirs) Add2Dots(cont);

  cDirEntry tde;

  tde.isdir=false;

  do {

    ShowRotator();

    char namebuf[1024];

    unz_file_info uzfi;

    if (unzGetCurrentFileInfo(

      uzp,     //the zipfile

      &uzfi,   //fileinfo structure

      namebuf, //buffer for the filename

      sizeof(namebuf),  //and size of the above

      NULL,0,  //we don't want extrainfo

      NULL,0   //and we dont want comment info

      )==UNZ_OK) {



      string s=tde.name=namebuf;

      //tde.name=s;

      bool addfile;

      if (addfile=FixDirsInArchive(tde.name,spd)) {

        tde.MakeNormal(tde.name,("{"+spd.fullname+"}"+s),useAttr,

                       uzfi.external_fa,MakeTimeFromUZT(uzfi.tmu_date),

                       uzfi.uncompressed_size);

        tde.time=MakeTimeFromUZT(uzfi.tmu_date); //Hopefully faster*/

      }



      /*Handle the ugly archives with no dirrecs*/

      if (tde.isdir) nodirrecs=false; //Hey, we've got dirrecs!

      if (!tde.isdir  //if we have a file

         && nodirrecs //and no dirrecs

         && (s.find('/')!=-1))  //and it's not in the root of the archive

           CheckForMissingDirRecord(cont,s,spd, useAttr); //Check and perhaps insert proper dir

      /******************************************/

           

      if (addfile) { //Only add if confirmed previously

        if (onlydirs) {

           if (tde.isdir) cont+=tde;

        } else cont+=tde;

      }

    }

  } while (unzGoToNextFile(uzp)!=UNZ_END_OF_LIST_OF_FILE);

  unzClose(uzp);

  return true;

}





//Used for typecasting unzFile-pointer so I can access position of physical file

typedef struct

{

	FILE* file;                 /* io structore of the zipfile */

	unz_global_info gi;       /* public global information */

	uLong byte_before_the_zipfile;/* byte before the zipfile, (>0 for sfx)*/

	uLong num_file;             /* number of the current file in the zipfile*/

	uLong pos_in_central_dir;   /* pos of the current file in the central dir*/

	uLong current_file_ok;      /* flag about the usability of the current file*/

	uLong central_pos;          /* position of the beginning of the central dir*/



	uLong size_central_dir;     /* size of the central directory  */

	uLong offset_central_dir;   /* offset of start of central directory with

                                       respect to the starting disk number */



//	unz_file_info cur_file_info; /* public info about the current file in zip*/

//	unz_file_info_internal cur_file_info_internal; /* private info about it*/

//      file_in_zip_read_info_s* pfile_in_zip_read; /* structure about the current

//	                                    file if we are decompressing it */

} unz_s;







bool UnpackZIP(const string& filename, const string& destdir, tProgFunc progress ) {

  long zipsize=GetFileSize(filename);

  unzFile uzp=unzOpen(filename.c_str());

  if (!uzp) return false;

  cFile* outf;

  do {

    char namebuf[1024];

    unz_file_info uzfi;

    if (unzGetCurrentFileInfo(

      uzp,     //the zipfile

      &uzfi,   //fileinfo structure

      namebuf, //buffer for the filename

      sizeof(namebuf),  //and size of the above

      NULL,0,  //we don't want extrainfo

      NULL,0   //and we dont want comment info

      )==UNZ_OK) {

      string s=namebuf;

      int fileattr=uzfi.external_fa;

      if ((fileattr & 0x10)==0x10) {

        MkDir(destdir+s);

      } else {

        if (unzOpenCurrentFile(uzp)!=UNZ_OK) {

          unzClose(uzp);

          return false;

        }

        MakeComplexDir(destdir+s,true);//Try to make sure path exists

        outf=GetFilePointer(destdir+s);

        if (!outf->Open(omWrite)) {

          scDELETE(outf);

          unzClose(uzp);

          return false;

        }

        int read; char buf[65536];

        do {

          read=unzReadCurrentFile(uzp,&buf,sizeof(buf));

          if (read>0) outf->Write(&buf,read);

          long zippos=ftell(((unz_s*)uzp)->file);

          progress(zippos,zipsize,0);

        } while (read>0);

        unzCloseCurrentFile(uzp);

        outf->SetMode(fileattr);

        outf->Close(); scDELETE(outf);

      }

    }

  } while (unzGoToNextFile(uzp)!=UNZ_END_OF_LIST_OF_FILE);

  unzClose(uzp);

  return true;

}





