/***************************************************************************
                          arc_zip.h  -  description
                             -------------------
    begin                : Sun Aug 20 2000
    copyright            : (C) 2000 by Morten Poulsen
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef _ARC_ZIP_
#define _ARC_ZIP_

#include "zip.h"
#include "unzip.h"
#include "sc_file.h"
#include "fm_vfs.h"
#include "fm_defs.h"
#include "fm_window.h"
#include "sc_action.h"

class cFileInZIP : public cFile {
  public:
    cFileInZIP(const sParsedDir& spd) : cFile(spd.fullname) {

      uzf=NULL;
      zpos=0;
      parsed=spd;
    }
    ~cFileInZIP() {
      switch (mode) {
        case omGZRead :
        case omRead   : unzClose(uzf); break;
        default : zipClose(zf,NULL); //No global comment
      }
      //if (uzf) delete uzf;
    }

    bool Open(tOpenMode om) {
      mode=om;
      //sParsedDir spd=PleaseParseDir(filename);
      switch (om) {
        case omGZRead :
        case omRead   : {
          uzf=unzOpen(parsed.fullname.c_str());
          if (!uzf) return false;
          if (unzLocateFile(uzf,parsed.arc_fullname.c_str(),0)!=UNZ_OK) return false;
          //0=OS-specific case sensitivity
          if (unzGetCurrentFileInfo(
                uzf,     //the zipfile
                &uzfi,   //fileinfo structure
                NULL,0,   //we already have the name
                NULL,0,  //we don't want extrainfo
                NULL,0   //and we dont want comment info

          )!=UNZ_OK) return false;
          if (unzOpenCurrentFile(uzf)!=UNZ_OK) return false;
          //phew a lot of checks, but we should be ready to rock!
          break;
        }

        case omGZWrite :
        case omWrite : {
          zf=zipOpen(parsed.fullname.c_str(),1); //append=1=yes
          if(!zf) return false;
          if(!zipOpenNewFileInZip(zf, //our "mother"-file
                                  parsed.arc_fullname.c_str(), //file to be packed
                                  &zfi, //fileinfo
                                  NULL,0, //no local extrafield
                                  NULL,0, //no global extrafield
                                  NULL, //No comment
                                  Z_DEFLATED, //pack, not store
                                  Z_DEFAULT_COMPRESSION)); //Default compression


          return false;
          break;
        }
      }
      return true;
    }
    int Read(void *buf,int size) {
      return unzReadCurrentFile(uzf,buf,size);
    }
    int Write(void *buf,int size) {
      return zipWriteInFileInZip(zf,buf,size);
    }
    void Close() {
      switch (mode) {
        case omGZRead :
        case omRead : unzCloseCurrentFile(uzf); break;
        default : zipCloseFileInZip(zf);
      }
    }
    bool Delete() {
      return false;
    }
    int Size() {
      return uzfi.uncompressed_size;
    }
    int Pos() {
      return unztell(uzf);
    }
    int GetMode() {
      return uzfi.external_fa;
    }
    bool SetMode(int) {
      return false;
    }

  private:
    union {
      unzFile uzf;
      zipFile zf;
    };
    union {
      unz_file_info uzfi;
      zip_fileinfo zfi;
    };
    int zpos;
    sParsedDir parsed;
};



bool ZIPScan(tDirContainer& cont, const sParsedDir& spd, bool onlydirs=false);
bool UnpackZIP(const string& filename, const string& destdir, tProgFunc progress);


#endif //_ARC_ZIP








