/***************************************************************************
             fm_conf.cpp  -  Handles configurationfiles of SC
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fm_conf.h"
#include "inifile.h"
#include "fm_defs.h"
#include "fm_str.h"
#include "fm_menu.h"
#include "fm_editcon.h"

#include "target.h"
#ifdef _TARGETFLAVOR_UNIX_
  char _config_filename[] = ".scrc";
  string _default_tempdir = ".sctemp";
#endif

#ifdef _TARGETFLAVOR_DOS_
  char _config_filename[] = "c:/sc.ini";
  string _default_tempdir = "c:/.sctemp";
#endif

bool _conffile_exist=true;

cFmConfig Configuration;

string cFmConfig::GetConfigFilename() {
  string s;
#ifdef _TARGETFLAVOR_UNIX_
  s=getenv("HOME");
  s=(AddSlash(s)+_config_filename);
#else
  s=_config_filename;
#endif
  return s;
}

void cFmConfig::MakeDefaultMenus() {
  #ifdef _TARGETFLAVOR_UNIX_
    sMenuOption sm;
    sm.showline="Edit my .~bashrc"; sm.type=maEdit; sm.commandline="~/.bashrc";
    menuopts+=sm;

    sm.showline="View ~processor info"; sm.type=maView; sm.commandline="/proc/cpuinfo";
    menuopts+=sm;
  #else
    sMenuOption sm;
    sm.showline="Edit my ~AUTOEXEC.BAT"; sm.type=maEdit; sm.commandline="C:/AUTOEXEC.BAT";
    menuopts+=sm;

    sm.showline="Edit my ~CONFIG.SYS"; sm.type=maEdit; sm.commandline="C:/CONFIG.SYS";
    menuopts+=sm;
  #endif
}

void cFmConfig::MakeDefaultExt() {

}

#ifdef _FTP_POSSIBLE_
bool ParseFtpSite(const string& line, cFtpSite& site) {
  string s=line;
  /*int i=s.find(',');
  if (i==-1) return false;
  site.showname=s.substr(0,i); s=s.substr(i+1,999);
  */
  int i=s.find(',');
  if (i==-1) return false;
  site.ftpsite=s.substr(0,i); s=s.substr(i+1,999);

  i=s.find(',');
  if (i==-1) return false;
  site.initialdir=s.substr(0,i); s=s.substr(i+1,999);

  i=s.find(',');
  if (i==-1) return false;
  site.username=s.substr(0,i); s=s.substr(i+1,999);

  i=s.find(',');
  if (i==-1) return false;
  site.password=s.substr(0,i); s=s.substr(i+1,999);

  if (s!="passive") site.passivemode=false; else site.passivemode=true;
  
  return true;
}
#endif

bool ParseMenu(const string& line, sMenuOption& sm) {
  int i=line.find(',');

  if (i>-1) {
    string s=line;
    sm.showline=s.substr(0,i);
    s=s.substr(i+1,999);
    sm.type=maJustRun; sm.commandline=s; //defaults to simply run a program
    if (s.find("[edit]")==0) {
      sm.type=maEdit;
      sm.commandline=s.substr(6,999);
    };
    if (s.find("[view]")==0) {
      sm.type=maView;
      sm.commandline=s.substr(6,999);
    };
    if (s.find("[multi]")==0) {
      sm.type=maMulti;
      sm.commandline=s.substr(7,999);
    };
    if (s.find("[capture]")==0) {
      sm.type=maCapture;
      sm.commandline=s.substr(9,999);
    }
    return true;
  }
  return false;
}

bool cFmConfig::ReadConfigFile() {
  #ifdef _FTP_POSSIBLE_
    ftpsites.Clear();
  #endif
  menuopts.Clear();
  string s=GetConfigFilename();
  cINIFile cf(s.c_str(), faRead);
  if (cf.Error()) _conffile_exist=false;

 //Also used in case of errors for assignment of default values
  usesimplechars=cf.GetValue("UseSimpleChars",false);

  //Read linesetup
  showsize=cf.GetValue("ShowSize",true);
  showattr=cf.GetValue("ShowAttr",true);
  showdate=cf.GetValue("ShowDate", false);
  minnamelength=cf.GetValue("MinNameLength",16);
  //read QuickDirs
#ifdef _TARGETFLAVOR_UNIX_
  quickdirs[0]=getenv("HOME");
  for(int i=1; i<_NUMBEROFQUICKDIRS; i++) {
#else
  for(int i=0; i<_NUMBEROFQUICKDIRS; i++) {
#endif
    quickdirs[i]=cf.GetValue("QuickDir"+MkStr(i),"");
  }

  //Read UserMenu
  s=""; s=cf.GetValue("MENU",s);
  sMenuOption sm;
  while (s!="") {
    if (ParseMenu(s,sm)) {
      menuopts+=sm;
    }

    s=cf.GetValue("MENU","");
  }
  if (menuopts.size()==0) MakeDefaultMenus();

  //editor options
  useexternaleditor=cf.GetValue("UseExternalExitor",false);
  externaleditor=cf.GetValue("ExternalEditor","mcedit");
  tabsize=cf.GetValue("Tabsize",8);

  //read Miscellanous values
  waitafterexec=cf.GetValue("WaitAfterExec",true);
  alwaysrecursesubdirs=cf.GetValue("AlwaysRecurseSubdirs",true);
  deletebackups=cf.GetValue("DeleteBackups",true);
  gotolastdir=cf.GetValue("GotoLastDir",true);
  showtipofday=cf.GetValue("ShowTipOfDay",true);
  clockincorner=cf.GetValue("ClockInCorner",true);
  monochromesupport=cf.GetValue("MonochromeSupport",false);
  #ifdef _TARGETFLAVOR_UNIX_
    rootwarn=cf.GetValue("RootWarn",true);
  #endif

  #ifdef _FTP_POSSIBLE_
  //Read ftp settings
  anon_password=cf.GetValue("AnonymousPassword","scom@inter.net");
  s=""; s=cf.GetValue("FTPSITE","");
  cFtpSite site;
  while (s!="") {
    if (ParseFtpSite(s,site)) {
      ftpsites+=site;
    }
    s=cf.GetValue("FTPSITE","");
  }
  #endif

  if (cf.Error()) return false; else return true;
}

bool cFmConfig::WriteConfigFile() {
  string s=GetConfigFilename();
  cINIFile cf(s.c_str(), faWrite);
  cf.SetComment("#----------------------------------------#");
  cf.SetComment("#Configurationfile for Sunshine Commander#");
  cf.SetComment("#----------------------------------------#");
  cf.SetComment("Note: This file will be reformatted each time SC is run, don't try");
  cf.SetComment("to comment it yourself ;-)");

  //Write visual setup
  cf.SetComment();
  cf.SetComment("*** Visual setup ***");
  cf.SetComment("Should SC avoid high ASCII lines, dots etc.?");
  cf.SetValue("UseSimpleChars",usesimplechars);
//  cf.SetComment();
  cf.SetComment("Do you want to see Tip of the Day when program starts?");
  cf.SetValue("ShowTipOfDay", showtipofday);
//  cf.SetComment();
  cf.SetComment("Do you want to see the current time in the bottom corner of the screen?");
  cf.SetValue("ClockInCorner", clockincorner);
//  cf.SetComment();
  cf.SetComment("Should SC try look decent in monochrome consoles?");
  cf.SetValue("MonochromeSupport", monochromesupport);

  //Write Linesetup
  cf.SetComment();
  cf.SetComment("*** LineSetup for filelist ***");
  cf.SetValue("ShowSize",showsize);
  cf.SetValue("ShowAttr",showattr);
  cf.SetValue("ShowDate",showdate);
  cf.SetValue("MinNameLength",minnamelength);

  //Write Quickdirs
  cf.SetComment();
  cf.SetComment("*** QuickDirs ***");
  for(int i=0; i<_NUMBEROFQUICKDIRS; i++) {
    string s="QuickDir";
    s=(s+MkStr(i));
    cf.SetValue(s,quickdirs[i]);
  }

  //Write Menuoptions
  cf.SetComment();
  cf.SetComment("*** UserMenuOptions ***");
  cf.SetComment("Syntax:");
  cf.SetComment("  MENU=nametoshow,programtorun [parameters...]");
  cf.SetComment("    Runs a program");
  cf.SetComment("  MENU=nametoshow,[edit]filetoedit");
  cf.SetComment("    Edits a specified file");
  cf.SetComment("  MENU=nametoshow,[view]filetoview");
  cf.SetComment("    Views a specified file");
  cf.SetComment("  MENU=nametoshow,[capture]programtorun [parameters...]");
  cf.SetComment("    Runs a programs, captures its output and views it");
  cf.SetComment("  MENU=nametoshow,[multi]programtorun %f");
  cf.SetComment("    Runs a program for each tagged file (filename inserted at %f)");
  cf.SetComment("General parameters:");
  cf.SetComment("  %f current file");
  cf.SetComment("  %d current directory");
  cf.SetComment("  %p prompt something and insert it");
  cf.SetComment("  %l name of a temporary file containing all taggedfiles/dirs");
  cf.SetComment("Menuoptions are presented in the order they appear in the configfile");
  for(int i=0; i<menuopts.Size(); i++) {
    s=menuopts[i].showline+",";
    switch (menuopts[i].type) {
      case maEdit : s+="[edit]"; break;
      case maView : s+="[view]"; break;
      case maCapture : s+="[capture]"; break;
      case maMulti : s+="[multi]"; break;
      default : ; // maJustRun
    }
    s+=menuopts[i].commandline;
    cf.SetValue("MENU",s);
  }

  //Write editor options
  cf.SetComment();
  cf.SetComment("*** Editor Options ***");
  cf.SetComment("Should SC use an external editor?");
  cf.SetValue("UseExternalEditor",useexternaleditor);
  cf.SetComment("Which external editor to use");
  cf.SetValue("ExternalEditor",externaleditor);
  cf.SetValue("Tabsize",tabsize);

  #ifdef _FTP_POSSIBLE_
  //Write FTP-settings
  cf.SetComment();
  cf.SetComment("*** FTP settings ***");
  cf.SetValue("AnonymousPassword",anon_password);
  for (int i=0; i<ftpsites.Size(); i++) {
    cf.SetValue("FTPSITE",
      //      ftpsites[i].showname+","+
            ftpsites[i].ftpsite+","+
            ftpsites[i].initialdir+","+
            ftpsites[i].username+","+
            ftpsites[i].password+","+
            (ftpsites[i].passivemode ? "passive" : "nonpassive"));
  }
  #endif

  //Write Miscillanous info
  cf.SetComment();
  cf.SetComment("*** Miscellanous Values ***");
  cf.SetComment("Should SC wait for a keypress after executing external programs?");
  cf.SetValue("WaitAfterExec",waitafterexec);
//  cf.SetComment();
  cf.SetComment("Should SC always recurse subdirs without asking?");
  cf.SetValue("AlwaysRecurseSubdirs",alwaysrecursesubdirs);
//  cf.SetComment();

  cf.SetComment("Should SC delete backup of files after editor is correctly exited?");
  cf.SetValue("DeleteBackUps",deletebackups);
//  cf.SetComment();
  cf.SetComment("Should we change to current dir at exit?");
  cf.SetValue("GotoLastDir", gotolastdir);
//  cf.SetComment();

  #ifdef _TARGETFLAVOR_UNIX_
//    cf.SetComment();
    cf.SetComment("Should SC warn you if you are root?");
    cf.SetValue("RootWarn",rootwarn);
  #endif
  cf.SetComment();
  cf.SetComment("End of Sunshine Commander Configuration");

  if (cf.Flush()) return true; else return false;
}


void cFmConfig::EditConfiguration() {
  if (!_conffile_exist) {
    Configuration.WriteConfigFile();
    _conffile_exist=true;
  }
  cMenu m("Configuration");
  m+="~Edit configuration file";
  m+="~Reload configuration file";
  m+="~Save configuration file now";
  m+="e~Xit";
  int i;
  int defmen=1;
  do {
    i=m.Run(defmen);
    switch (i) {
      case 4 :
      case 0 : break;
      case 2 : Configuration.ReadConfigFile(); break;
      case 3 : Configuration.WriteConfigFile(); break;
      default : {
        if (DedicatedEdit(GetConfigFilename())) {
          Configuration.ReadConfigFile();
          WriteTopMessage("Configurations reread from file...");
          defmen=4;
        }
      }
    }
  } while (i!=0 && i!=4);
}
