/***************************************************************************
            fm_conio.cpp  -  handles the lowlevelwriting to screen
                          -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <time.h>
#include "fm_conio.h"
#include "fm_conf.h"
#include "fm_defs.h"
#include "fm_str.h"

#include "target.h"

#define _HLINE ""
#define _SIMPLE_HLINE "-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------"

#ifdef _TARGET_TEXTIO_CURSES_
  #include <curses.h>
  #undef erase //()
  #define _VLINE ACS_VLINE
  #define _HOLINE ACS_HLINE
  #define _TOPLEFT ACS_ULCORNER
  #define _TOPRIGHT ACS_URCORNER
  #define _BOTTOMLEFT ACS_LLCORNER
  #define _BOTTOMRIGHT ACS_LRCORNER
  #define _TITLE_LEFT ACS_LTEE
  #define _TITLE_RIGHT ACS_RTEE
#endif

#ifdef _TARGET_TEXTIO_CONIO_
  #define _VLINE ''
  #define _HOLINE ''
  #define _TOPLEFT ''
  #define _TOPRIGHT ''
  #define _BOTTOMLEFT ''
  #define _BOTTOMRIGHT ''
  #define _TITLE_LEFT ''
  #define _TITLE_RIGHT ''
#endif

#define _SIMPLE_VLINE '|'
#define _SIMPLE_HOLINE '-'
#define _SIMPLE_TOPLEFT '/'
#define _SIMPLE_TOPRIGHT '\\'
#define _SIMPLE_BOTTOMLEFT '\\'
#define _SIMPLE_BOTTOMRIGHT '/'
#define _SIMPLE_TITLE_LEFT '['
#define _SIMPLE_TITLE_RIGHT ']'

#define _FILLCHAR ''
#define _SIMPLE_FILLCHAR ' '
#define _TAGGEDCHAR ''
#define _SIMPLE_TAGGEDCHAR '*'

bool _initialized = false;

sBox GetBoxChars() {
  sBox temp;
  if (Configuration.usesimplechars) {
    temp.vline=_SIMPLE_VLINE; temp.hline=_SIMPLE_HOLINE;
    temp.upleft=_SIMPLE_TOPLEFT;       temp.upright=_SIMPLE_TOPRIGHT;
    temp.dnleft=_SIMPLE_BOTTOMLEFT;    temp.dnright=_SIMPLE_BOTTOMRIGHT;
    temp.titleleft=_SIMPLE_TITLE_LEFT; temp.titleright=_SIMPLE_TITLE_RIGHT;
    temp.fillchar=_SIMPLE_FILLCHAR; temp.taggedchar=_SIMPLE_TAGGEDCHAR;
  } else {
    temp.vline=_VLINE; temp.hline=_HOLINE;
    temp.upleft=_TOPLEFT;       temp.upright=_TOPRIGHT;
    temp.dnleft=_BOTTOMLEFT;    temp.dnright=_BOTTOMRIGHT;
    temp.titleleft=_TITLE_LEFT; temp.titleright=_TITLE_RIGHT;
    temp.fillchar=_FILLCHAR; temp.taggedchar=_TAGGEDCHAR;
  }
  return temp;
}

void WriteAt(const string& s, int x, int y, int attr) {
  if (_initialized) ConOut(x, y, s.c_str(),attr);
}

void WriteAt(const char s[], int x, int y, int attr) {
  if (_initialized) ConOut(x, y, s,attr);
}

void WrFast(const char s[],int x, int y, int fg,int bg) {
 if (_initialized) ConOut(x, y, s,fg+(16*bg));
}

void Wr(int x, int y, const string& s, int fg/*=_DEFAULTFG*/, int bg/*=_DEFAULTBG*/,int fill/*=0*/) {
  string temp;
  if (s=="") {
    if (fill>0) temp=GetSpaces(fill); else return;
  } else {
    temp=s;
    if (fill!=0 && s.length()<fill) temp+=GetSpaces(fill-s.length());
    if (x==_centered) x=(ScreenW()-s.length()) / 2;
  }
  if (temp.length()+x>ScreenW()) temp=LCopy(temp,ScreenW()-x+1);

  WriteAt(temp,x,y,fg+(16*bg));
}

void WrHl(int x, int y, const string& s, int fg/*=_DEFAULTFG*/, int bg/*=_DEFAULTBG*/,int fill/*=0*/) {
  int i;
  if (i=s.find('~')==-1) {
    //no chars highlightet, write fast!
    Wr(x,y,s,fg,bg,fill);
    return;
  }
  if (s.substr(i+1,9999).find('~')==-1) {
    //only 1 ~ should be reasonably fast anyway
    string temp=s;
    temp.erase(i,1);
    Wr(x,y,temp,fg,bg,fill);
    temp=s[i+1];
    Wr(x+i,y,temp,chYELLOW,bg);
    return;
  }
  //slow as hell, do not use more than 1 ~ unless you have to...
  static char buf[]="0"; //nullterminated
  for (int i=0; i<s.length(); i++) {
    if (s[i]=='~') {
      buf[0]=s[i+1];
      WriteAt(buf,x+i,y,chYELLOW+(bg*16));
      i++; x--;
    } else {
      buf[0]=s[i];
      WriteAt(buf,x+i,y,fg+(bg*16));
    }
  }

}

void ClearScreen() {
  if (_initialized) ConCS(CONCOLOR(cLGRAY,_DEFAULTBG));
}

void ClearScreenBlack() {
  if (_initialized) ConCS(CONCOLOR(cLGRAY,cBLACK));
}

void ClearScreenWhite() {
  if (_initialized) ConCS(CONCOLOR(cLGRAY,cWHITE));
}

string GetHLine(int len) {
  string temp;
  if (Configuration.usesimplechars) {
    temp.append(_SIMPLE_HLINE,1,len);
  } else {
    temp.append(_HLINE,1,len);
  }
  return temp;
}

void InitConIO() {
  ConInit();
  HideCursor();
  _initialized=true;
}

void DoneConIO() {
  _initialized=false;
  ShowCursor();
  ConDone();
}

void RefreshConIO() {
  ConRefresh();
}

void HideCursor() {ConCHide();};
void ShowCursor() {ConCShow();};

int ScreenW() {return ConMaxX();};
int ScreenH() {return ConMaxY();};

void GotoXY(int x,int y) {
  ConXY(x,y);
}

void WriteTopMessage(const string& s) {
  Wr(1,1,s,chYELLOW,cBLUE,ScreenW());
}

void ShowTime() {
  time_t x;
  tm *t;
  time(&x);
  t = gmtime(&x);
  string s=(Leading0(MkStr(t->tm_hour),2)+":"+
            Leading0(MkStr(t->tm_min),2)+":"+
            Leading0(MkStr(t->tm_sec),2));
  Wr(ScreenW()-8,ScreenH(),s,cWHITE,_DEFAULTBG);
}

