/***************************************************************************
                          fm_dircon.cpp  -  description
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "target.h"
#ifdef _SYMLINKS_EXIST_
  #include <unistd.h>
#endif

#include <stdio.h>
//#include <time.h>
#include "fm_sysinfo.h"
#include "fm_dircon.h"
#include "fm_conf.h"
#include "fm_fileinfo.h"
#include "fm_conio.h"
#include "fm_tree.h"
#include "fm_menu.h"
#include "fm_entry.h"
#include "fm_scd.h"
#include "sc_shell.h"
#include "fm_debug.h"
#include "sc_action.h"
#include "fm_helpscreens.h"
#include "sc_file.h"
#include "fm_defs.h"

#ifdef _TARGETFLAVOR_DOS_
  #include <dos.h>
  const int _firsttruefile=2; //Index (0 start) of first "normal" file in the files[]
#endif

#ifdef _TARGETFLAVOR_UNIX_
  #include <sys/stat.h>
  const int _firsttruefile = 3; //Index (0 start) of first "normal" file in the files[]
#endif

#define NORMALENTRY (!files[cur].isspecial && files[cur].name!="..")
#define NORMALFILE (!files[cur].isdir)

const int _sizelength=12;
const int _datelength=18;//hh:mm:ss dd:mm:yy

cArray <cDirEntry> fcollection;

//Prototypes, implementations last in this file
void HandleCollection(cDirEntry& de);
void RemoveFromCollection(const cDirEntry& de);

void cDirWindow::CheckCollection() {
  if (fcollection.Size()==0) return;
  for (int i=0; i<files.Size(); i++) {
    for (int ii=0; ii<fcollection.Size(); ii++) {
      if (files[i].fullname==fcollection[ii].fullname) SET(files[i].colltagged);
    }
  }
}

cDirWindow::cDirWindow() {
  topfile=0;
  cur=0;
  GetBriefFilesystemInfo(filesys);
  box=GetBoxChars();
  mask="*";
  Dbg("fm_dircon: Initializing DirConsole");
  fullpath=CurrentDir();
  lastdir=CurrentDir(); //initial dir first in history
  Rescan();
}

cDirWindow::~cDirWindow() {
}

void cDirWindow::Idle() {
  if (Configuration.clockincorner) ShowTime();

}


string cDirWindow::MakeFileString(int file) {
  string s=files[file].name;
  if (files[file].isdir || files[file].isftpsite) {
    s=('['+s+']');
  }
  if (files[file].isspecial) {
    return LCopy(s,linelength-1);
  }

#ifdef _SYMLINKS_EXIST_
  if (files[file].islink) {
    if (files[file].badlink) s=('!'+s); else s=('@'+s);
  }
#endif

  if (files[file].isarchive) {
    s=('{'+s+'}');
  }

  if (files[file].colltagged) {
    s=('#'+s);
  }

  if (Configuration.showsize) {
    if (files[file].isdir || files[file].isftpsite) { //No size for dirs/ftpsite
      s=LCopy(s,namelength+_sizelength);
    } else {
      s=LCopy(s,namelength);
      s+=RCopy(MkStr(files[file].size,true),_sizelength);
    }
  } else {
    s=LCopy(s,namelength);
  }
  if (Configuration.showattr) {
    s+=files[file].GetFileModeStr();
  }
  return s;
}

string cDirWindow::GetTitle() {
  if (IsDirDir(fullpath)) {
    return "Dir: "+AddSlash(fullpath);
  } else {
    return "Dir: "+fullpath;
  }
}

void cDirWindow::CalcFilelistInfo() {
  #ifdef _TARGETFLAVOR_DOS_
    int _attrlength=5; bool _useattr=true;
  #else
    int _attrlength=10; bool _useattr=false;
  #endif
  for(int i=1; i<files.Size(); i++) {
    if (!files[i].isspecial) {
      _attrlength=files[i].GetFileModeStr().length();
      _useattr=files[i].useattr;
      break;
    }
  }
  //This part is ugly, but it works great ,-)
  linelength=1; //1=space for tagged indicator
  if (Configuration.showsize) linelength+=_sizelength;
  if (Configuration.showattr) linelength+=_attrlength;
  if (Configuration.showdate) linelength+=_datelength;
  nocolumns=1;
  while (nocolumns*(linelength+Configuration.minnamelength)<ScreenW()) nocolumns++;
  nocolumns--;
  int i=nocolumns*(linelength+Configuration.minnamelength);
  namelength=((ScreenW()-i) / nocolumns)+Configuration.minnamelength;
  linelength+=namelength;
  linespercolumn=ScreenH()-4; //1 title, 2 headers, 3 currentfile, 4 filetotals
  totalfilesonscreen=linespercolumn*nocolumns;

  titleline=" "+LCopy("Name",namelength);
  if (Configuration.showsize) titleline+=RCopy("Size", _sizelength);
  if (_useattr) {
    if (Configuration.showattr) titleline+=RCopy("Attr", _attrlength);
  } else {
    if (Configuration.showattr) titleline+=RCopy("Mode", _attrlength);
  }
  if (Configuration.showdate) titleline+=RCopy("Date", _datelength);
}


void cDirWindow::WriteHeader() {
  Wr(3,1,GetTitle(),chYELLOW,cBLUE,ScreenW());
  for(int i=0; i<nocolumns; i++) {
    Wr(i*linelength+1,2,titleline,chYELLOW,cBLUE);
  }
  string s="[Marked: ";
  s=s+MkStr(CM())+" files "+MkStr(MM(),true)+" bytes] [Total: ";
    s=s+MkStr(CA())+" files "+MkStr(MA(),true)+" bytes]";
  Wr(1,ScreenH(),s,chYELLOW,cBLUE,ScreenW()-1);
}

void cDirWindow::Write1File(int f) {
  int column=(f-topfile) / linespercolumn;
  int fx=column*linelength+1;
  int fy=((f-topfile) % linespercolumn)+3;
  if (f>=(CA()+NOS())) {
    Wr(fx,fy,GetSpaces(linelength));
    return;
  }
  string s=MakeFileString(f);
  int bg=cBLUE;
  int fg=cLGRAY;


  if (files[f].isdir || files[f].isexecutable) fg=chWHITE;
  if (files[f].isarchive) fg=cGREEN;
  if (files[f].isspecial) fg=chYELLOW;
  if (files[f].isftpsite) fg=chRED;

  #ifdef _SYMLINKS_EXIST_
    if (files[f].islink) {
      if (files[f].badlink) {
        fg=cYELLOW;
      } else {
        fg=cGREEN;
      }
    }
  #endif

  if (files[f].colltagged) {
    fg=cCYAN;
  }

  if (files[f].tagged) {
    s=box.taggedchar+s;
    if (f==cur) fg=chWHITE; else fg=cRED;
  } else s=' '+s;

  if (f==cur) bg=cRED;

  if (Configuration.monochromesupport && MonochromeConsole()) {
    if (f==cur) {
      if (files[f].tagged) s[0]='$'; else s[0]='>';
    }
  }

  Wr(fx,fy,s,fg,bg,linelength);

  if (f==cur) {
    if (!files[cur].isdir) {
      s=files[cur].name;
    } else {
      s=("["+files[cur].name+"]");
    }
    #ifdef _SYMLINKS_EXIST_
      if (files[cur].islink) {
        s=files[cur].name;
        s+=" -> "+files[cur].linkto;
        if (files[cur].badlink) s+=" (bad)";
      }
    #else

    #endif
    string s2=files[cur].GetFileTimeStr();
    
    if (files[cur].isdir) {
      s2+=" [DIR]";
    } else {
      s2+=files[cur].GetFileModeStr();
    }
    s=(LCopy(s, ScreenW()-s2.length())+s2);
    Wr(1,ScreenH()-1,s,cCYAN,cBLACK,ScreenW());
    if (filesearch!="") {
      Wr(1,ScreenH()-1,s.substr(0,filesearch.length()),chWHITE,cCYAN);
    }
  }
}

void cDirWindow::WriteAllFiles() {
  for(int i=topfile; i<(totalfilesonscreen+topfile); i++) {
     Write1File(i);
  }
}

void cDirWindow::MoveToFile(const string& fname) {
  for(int i=0; i<files.Size(); i++) {
    if (Equal(files[i].name,fname)) {
      cur=i;
      UpdateFirstShown();
    }
  }
}


void cDirWindow::UpdateFirstShown() {
  while ((topfile+totalfilesonscreen-1) < cur) topfile+=linespercolumn;
  while (topfile>cur) topfile-=linespercolumn;
}

void cDirWindow::TreatMoveKey(tKey k) {
  filesearch="";
  int oldcur=cur;
  switch (k) {
    case kBlindDown : {if (cur<files.Size()-1) cur++;return;}
    case kBlindUp : {if (cur>0) cur--; return;}
  }
  switch (k) {
    case kUp : TreatMoveKey(kBlindUp); break;
    case kDown : TreatMoveKey(kBlindDown); break;
    case kLeft : for(int i=0; i<linespercolumn; i++) TreatMoveKey(kBlindUp); break;
    case kRight : for(int i=0; i<linespercolumn; i++) TreatMoveKey(kBlindDown); break;
    case kPgup : for(int i=0; i<totalfilesonscreen; i++) TreatMoveKey(kBlindUp); break;
    case kPgdn : for(int i=0; i<totalfilesonscreen; i++) TreatMoveKey(kBlindDown); break;
    case kHome : cur=0; break;
    case kEnd  : cur=files.Size()-1; break;
    case kSpace : {
      /*if (fullpath==_FTPROOTNAME) break;
      if (files[cur].name[0]!='/' &&
          files[cur].name!=".." &&
          files[cur].name[1]!=':')*/
      if (files[cur].taggable) {
        INVERT(files[cur].tagged);
        WriteHeader();
        TreatMoveKey(kBlindDown);
      }
      break;
    }
    default: break;
  }
  int oldtopfile=topfile;
  UpdateFirstShown();
  if (oldtopfile!=topfile) { //rewrite all
    WriteAllFiles();
  } else {
    if (oldcur!=cur || cur==files.Size()-1) { //rewrite only 2 files
      Write1File(oldcur);
      Write1File(cur);
    }
  }
}

void cDirWindow::TreatKey(tKey k) {
  if (KeyIn(k,kUp,kDown,kPgup,kPgdn,kLeft,kRight,kHome,kEnd,kSpace)) {
    TreatMoveKey(k);
    return;
  }
  if (KeyIn(k,_KEYSET_ALTLETTERS)) {
    SearchFile();
    return;
  }
  #ifdef _TARGETFLAVOR_DOS_
  if (KeyIn(k,_KEYSET_CTRLLETTERS)) {
    string drive="";
    drive=Key_LastASCIIChar(); drive+=":/";
    SetNewDir(drive);
    return;
  }
  #endif
  if (KeyIn(k,_KEYSET_NUMBERS)) {
    if (Configuration.quickdirs[Key_LastNumber()]!="") {
      if (DoesDirExist(Configuration.quickdirs[Key_LastNumber()])) {
        fullpath=Configuration.quickdirs[Key_LastNumber()];
        Rescan();
        Update();
      }
    }
    return;
  }
  if (KeyIn(k,_KEYSET_ALTNUMBERS)) {
    int i=Key_LastNumber();
    #ifdef _TARGETFLAVOR_UNIX_
      if (i==0) return; //0 taken for homedir
    #endif
    string s="Set quickdir ";
    s+=MkStr(i);
    s+=" to current dir? (~Y/N)";
    if(KeyIn(PromptBox("Confirmation",s,kEnter,kESC,kY,kN),kEnter,kY)) {
      Configuration.quickdirs[i]=fullpath;
    }
    Update();
    return;
  }
  bool refresh=true;
  switch (k) {
    case kEnter : Execute(); UpdateDirHistory(); break;
    case kTab : if (NORMALENTRY) {
      #ifdef _FTP_POSSIBLE_
      if (fullpath==_FTPTOKEN) {
        if (EditFtpSite(cur-_firsttruefile/*files[cur].name*/)) Rescan();
        break;
      }
      #endif
      if (files[cur].tagged) ShowFileInfo(files); else ShowFileInfo(files[cur]);
      break;
    }
    case kF5 : {
      #ifdef _TARGETFLAVOR_UNIX_
        ClearScreen(); //clear screen for kernelmessages etc.
      #endif
      string s=files[cur].name;
      Rescan();
      MoveToFile(s);
      break;
    }
    case kH :
    case kF1 : ShowDirHelp(); break;

    case kF7 : RunProgram(GetCurrentFile()); break;
    case kF4 : ShowUserMenu(files,GetCurrentFile(),GetFullPath()); break;

    case kSlash :
    case kBackspace :
    case kESC : {
      string s;
      switch (k) {
        case kSlash : s=FmTree->GetPath("Choose Directory - F1 for help",fullpath,"", false,true); break;
        case kESC : s=FmTree->GetPath("Choose Directory - F1 for help",fullpath); break;
        default : s=SCD_GetDir();
      }
      if (s!="") {
        if (DoesDirExist(s)) {
          fullpath=s;
          Rescan();
          UpdateDirHistory();
        }
      }
      break;
    }

    case kInsert : {
      #ifdef _FTP_POSSIBLE_
      if (fullpath==_FTPTOKEN) {
        string moveto;
        if (EnterNewFtpSite()) {
          Rescan();
          MoveToFile(moveto);
        }
        break;
      }
      #endif
      string s="";
      if (InputWindow("New dir:",s,ScreenW()-10)) {
        s=(AddSlash(fullpath)+s);
        if (MkDir(s)) {
          Rescan();
          MoveToFile(s);
        } else {
          ErrorBox("Could not create directory!");
        }
      }
      break;
    }

    case kF2 : {
      ShowSystemInfo();
      ClearScreen();
      Update();
      break;
    }

    case kL : {
      if (NORMALENTRY) {
        HandleCollection(files[cur]);
      }
      TreatMoveKey(kDown);
      break;
    }

    case kO : {
      if (fcollection.Size()>0) {
        cMenu m("FileCollection");
        m+="~Copy  filecollection";
        m+="~Edit  filecollection";
        m+="C~lear filecollection";
        int i=m.Run();
        switch (i) {
          case 1 : {
            cActionProject ap(fcollection,"",acCopy);
            if (!ap.Error()) ap.Doit();
            fcollection.Clear();
            Update();
            break;
          }
          case 2 : {
            MessageBox("Not Implemented!");
            break;
          }
          case 3 : {
            fcollection.Clear();
            break;
          }
          default : ;
        }
      } else {
        MessageBox("No files in collection");
      }
      break;
    }

    case kAltBackspace : {
      if (!dirhistory.Empty()) {
        if (dirhistory.Last()!=fullpath) {
          string d=dirhistory.PopStart();
          //d=dirhistory.PopStart();
         // if (d==CurrentDir()) d=dirhistory.PopEnd();
          if (SetDir(d)) {
            Rescan();
          }
        }
      }
      break;
    }

    case kF : {
      string filename="";
      string s=SCD_FindFile(filename);
      if (s!="") {
        if (SetDir(s)) {
          Rescan();
          MoveToFile(filename);
          UpdateDirHistory();
        }
      }
      break;
    }
    case kT : DoTagging(); break;
    case kZ : {
      Configuration.EditConfiguration();
      break;
    }

    //Copy, Erase, Move, Pack and Unpack
    case kA :
    case kC :
    case kDelete :
    case kE :
    case kM :
    case kP :
    case kU : {
      #ifdef _FTP_POSSIBLE_
      if (fullpath==_FTPTOKEN) {
        if (DeleteFtpSite(cur-_firsttruefile/*files[cur].name*/)) Rescan();
        break;
      }
      #endif
      if (NORMALENTRY || CM()>0) { //If currentfile or some tagged
        tActionType ac;
        switch (k) {
          case kA : ac=acAttr; break;
          case kC : ac=acCopy; break;
          case kM : ac=acMove; break;
          case kDelete :
          case kE : ac=acErase; break;
          case kP : ac=acPack;  break;
          case kU : ac=acUnpack;break;
        }
        bool error;
        if (CM()==0) {
          /*cActionProject ap(files,fullpath,ac);
          if (!ap.Error()) error=ap.Doit();
        } else {
          cActionProject ap(files[cur],fullpath,ac);
          if (!ap.Error()) {
            error=ap.Doit();
          }*/
          files[cur].tagged=true;
        }
        cActionProject ap(files,fullpath,ac);
        if (!ap.Error()) error=ap.Doit();
        if (Tree_GotoDestination!="") {
          if (DoesDirExist(Tree_GotoDestination)) {
            fullpath=Tree_GotoDestination;
            Rescan();
            UpdateDirHistory();
          }
        } else {
          int oldcur=cur;
          Rescan();
          if (oldcur>files.Size()) cur=files.Size()-1; else cur=oldcur;
          UpdateFirstShown();
        }
      }
      break;
    }


    case kS : {
      #ifdef _SYMLINKS_EXIST_
         if (files[cur].islink) {
           string s=files[cur].linkto;
           if (InputWindow("New target:",s,60)) {
             if (DeleteFile(files[cur].fullname)) {
               if (symlink(s.c_str(),files[cur].fullname.c_str())!=-1) {
                 files[cur].linkto=s; //success
               } else  ShowErrorMessage();
             } else ShowErrorMessage();
           }
         } else {
           static string lastposition="";
           string dir=FmTree->GetPath("Where should SymLink be placed?", fullpath,lastposition);
           dir=AddSlash(dir);
           if (dir!="") {
             string name=files[cur].name;
             if (InputWindow("What should SymLink be called?",name,50)) {
               name=(dir+name);
               if (symlink(files[cur].fullname.c_str(),name.c_str())==-1) {
                 ShowErrorMessage();
               }
             }
           }
         }
         files[cur].CheckLink();
      #endif
      break;
    }
    

    //Rename function, now combined with Duplicate
    case kD :
    case kR : {
      if (NORMALENTRY) { //see top for macro
        string s=files[cur].name;
        if (InputWindow("New name: ",s,ScreenW()-4)) {
          if ((files[cur].isdir ? DoesDirExist(s) : DoesFileExist(s))) {
            if (PromptBox("Name already exists!",
                          "Do you want to overwrite? (Y/~N))",
                          kY,kN,kESC,kEnter)!=kY) break;
          }
        } else break;
        bool result;
        switch (k) {
          case kR : result=(Rename(files[cur].fullname,AddSlash(fullpath)+s)); break;
          case kD : result=CopyFile(files[cur].fullname,AddSlash(fullpath)+s); break;
          default : ;
        }
        if (!result) {
          ShowErrorMessage();
        } else {
          Rescan();
          MoveToFile(s);
        }
      }
      break;
    }
      
    default : refresh=false; //don't refresh if unknown keypress
  }
  if (refresh) Update();
}


string cDirWindow::GetCurrentFile() {
  if (!files[cur].isdir
/*  #ifdef _TARGET_LINUX_
    && !files[cur].islink
  #endif*/
  ) {
    return files[cur].fullname;
  } else return "";
}

bool cDirWindow::Rescan() {
  Dbg("fm_dircon: Rescanning dir");
  filesearch="";

  //Try to scan for files
  if (ScanForFiles(files,AddSlash(fullpath))) {
    Dbg("fm_dircon: Adding special direntries");
    cDirEntry temp;
    #ifdef _TARGETFLAVOR_UNIX_
      //Adding home in linux
      temp.MakeSpecial("~",getenv("HOME"));
      files.Insert(temp,1);
      //and root
      if (fullpath!="/") {
        temp.MakeSpecial("/");
        files.Insert(temp,1);
      }
    #else
      //adding root in dos
      if (fullpath.length()>3) {
        temp.MakeSpecial("/");
        files.Insert(temp,1);
      }
    #endif
    //and adding filesystems/drives
    for (int i=0; i<filesys.Size(); i++) {
      if (filesys[i].mountpoint=="/") continue;
      temp.MakeSpecial(filesys[i].mountpoint);
      files+=temp;
    }
    #ifdef _FTP_POSSIBLE_
      //adding FTP-access
      temp.MakeFtpSite(_FTPTOKEN); files+=temp;
    #endif
    topfile=0; cur=0;

    //Check for collectiontagging
    CheckCollection();
    return true;
  } else {
    ErrorBox("Dir not available!");
    return false;
  }
  filesearch="";
  return false;
}

void cDirWindow::SetNewDir(const string& dir,const string& movetofile/*=""*/) {
  string savepath=fullpath; //For safety, if unsuccessful
  fullpath=dir;
  if (fullpath=="") fullpath="/";
  #ifdef _TARGETFLAVOR_DOS_
    if (fullpath.length()==2) fullpath=AddSlash(fullpath);
  #endif
  if (fullpath!="/") fullpath=StripSlash(fullpath);
  if (Rescan()) { //The new dir works great!
    if (movetofile!="") MoveToFile(movetofile);
    Update();
    return;
  } else { //the new dir does NOT work great, lets try the one we came from
    fullpath=savepath;
    if (Rescan()) {
      Update();
      return;
    } else { //Ooops, that didn't work, either, we must get a working dir...
      string s;
      do {
        do {
          s=FmTree->GetPath("Choose Directory - F1 for help",fullpath);
        } while (s==""); //We HAVE to get a new dir...
      } while(!Rescan());
      Update(); //phew, now it works...
      return;
    }
  }
}

void cDirWindow::Execute() {
  cDirEntry de=files[cur];
  if (de.name=="..") {
    string moveto,newdir;
    if (IsDirArc(fullpath)) {
      //The following is really ugly, maybe I'll shine it up some time

      sParsedDir spd=PleaseParseDir(fullpath,true);
      //fullpath=StripSlash(fullpath);
      if (spd.arc_dirname=="") {
        SetNewDir(spd.dirname,spd.filename);
        return;
      }
      spd.arc_fullname=StripSlash(spd.arc_fullname);
      int i=spd.arc_fullname.rfind('/');
      if (i!=-1) {
        moveto=spd.arc_fullname.substr(i+1,999);
      } else {
        moveto=StripSlash(spd.arc_dirname);
      }
      fullpath=StripSlash(fullpath);
      if (i==-1) {
        newdir=("{"+spd.fullname+"}");
      } else {
        newdir=fullpath.substr(0,fullpath.rfind('/')+1);
      }

      SetNewDir(newdir,moveto);
      return;
    }
    #ifdef _FTP_POSSIBLE_
    if (IsDirFtp(fullpath)) {
      sParsedDir spd=PleaseParseDir(fullpath,true);
      //fullpath=StripSlash(fullpath);
      if (spd.arc_dirname=="") {
        SetNewDir(_FTPTOKEN,spd.filename);
        return;
      }
      spd.arc_fullname=StripSlash(spd.arc_fullname);
      int i=spd.arc_fullname.rfind('/');
      if (i!=-1) {
        moveto=spd.arc_fullname.substr(i+1,999);
      } else {
        moveto=StripSlash(spd.arc_dirname);
      }
      fullpath=StripSlash(fullpath);
      if (i==-1) {
        newdir=("["+spd.fullname+"]");
      } else {
        newdir=fullpath.substr(0,fullpath.rfind('/')+1);
      }

      SetNewDir(newdir,moveto);
      return;
    }
    #endif

    //Normal dir
    moveto=fullpath.substr(fullpath.rfind('/')+1,999);
    newdir=fullpath.substr(0,fullpath.rfind('/')+1);
    SetNewDir(newdir,moveto);
    return;
  }

  if (de.name=="...") {
    sParsedDir spd=PleaseParseDir(fullpath);
    SetNewDir(spd.dirname,spd.filename);
    return;
  }

  if (de.name=="/") {
    #ifdef _TARGETFLAVOR_DOS_
      if (IsDirArc(fullpath)) {
        SetNewDir(fullpath.substr(1,3),""); //Skip archive breace Goto root of drive
      } else {
        SetNewDir(fullpath.substr(0,3),""); //Goto root of drive
      }
    #else
      SetNewDir("/","");
    #endif
    return;
  }
  if (de.isdir) {
    string todir=de.fullname;
    #ifdef _SYMLINKS_EXIST_
      if (de.islink && de.linkto!="")
        todir=de.linkto;
    #endif
    SetNewDir(todir);
    return;
  }
  if (de.isarchive) {
    if (DoesFileExist(de.fullname)) {
      SetNewDir("{"+de.fullname+"}");
    };
    return;
  }
  if (de.isftpsite) {
    SetNewDir(de.fullname);
    return;
  }

  if (de.isexecutable) {
    ExecuteCommand(de.fullname);
  }
}

void cDirWindow::SearchFile() {
  string temp=filesearch;
  temp+=Key_LastASCIIChar();
  for (int i=0; i<files.Size(); i++) {
    if(MatchStart(temp,files[i].name)) {
      filesearch=temp;
      cur=i;
      UpdateFirstShown();
      WriteAllFiles();
      return;
    }
  }
}

void cDirWindow::Update() {
  CalcFilelistInfo();
  WriteAllFiles();
  WriteHeader();
}

bool cDirWindow::DoTagging() {
  cMenu m("Tagging");
  m+="~All";
  m+="All + ~dirs";
  m+="~Several";
  m+="~Invert";
  m+="~Clear";
  int choice=m.Run();
  if (!choice) return true;

  string mask="*";
  if (choice==3) {
    if (!InputWindow("Mask to tag", mask, 16)) return true;
  }

  for (int i=0; i<files.Size(); i++) {
    if (files[i].name==".." || files[i].name=="/" || files[i].isspecial || !files[i].taggable) continue;
    switch (choice) {
      case 1 : {
        if (!files[i].isdir) SET(files[i].tagged);
        break;
      }
      case 2 : {
        SET(files[i].tagged);
        break;
      }
      case 3 : {
        if (WildcardMatch(files[i].name,mask)) SET(files[i].tagged);

        break;
      }
      case 4 : {
        INVERT(files[i].tagged);
        break;
      }
      case 5 : {
        CLEAR(files[i].tagged);
      }
      default : {
      }
    }
  }
  return true;
}

int cDirWindow::CA() {
  int c=0;
  for(int i=0; i<files.Size(); i++) {
    if (!files[i].isspecial) c++;
  }
  return c;
}

int cDirWindow::CM() {
  int c=0;
  for(int i=0; i<files.Size(); i++)
    if (!files[i].isspecial && files[i].tagged) c++;
  return c;
}

long int cDirWindow::MA() {
  long int c=0;
  for(int i=0; i<files.Size(); i++)
    c+=files[i].size;
  return c;
}

long int cDirWindow::MM() {
  long int c=0;
  for(int i=0; i<files.Size(); i++)
    if (files[i].tagged) c+=files[i].size;
  return c;
}

int cDirWindow::NOS() {
  int c=0;
  for(int i=0; i<files.Size(); i++)
    if (files[i].isspecial) c++;
  return c;
}

void cDirWindow::UpdateDirHistory() {
  dirhistory.PushStart(lastdir);
  lastdir=fullpath;
}

/*************************************************************************/
//COLLECTION
/*************************************************************************/

void HandleCollection(cDirEntry& de) {
  if (de.colltagged) {
    for (int i=0; i<fcollection.Size(); i++) {
      if (fcollection[i].fullname==de.fullname) {
        fcollection-=i;
        CLEAR(de.colltagged);
        return;
      }
    }
  } else {
    for (int i=0; i<fcollection.Size(); i++) {
      if (fcollection[i].fullname==de.fullname) return; //Don't add same twice
    }
    fcollection+=de;
    SET(fcollection.Last().tagged); //all should be tagged in collection
    SET(de.colltagged);
  }
}

void RemoveFromCollection(const cDirEntry& de) {
  if (fcollection.Size()==0) return;
  cWindow w(ScreenW()-10,ScreenH()-10,"Edit filecolletion");
  int topline=0;
  bool rewrite=true;
  tKey k;
  do {
    if (rewrite) {
      for (int i=topline; i<w.GetHeight()-2; i++) {
        string s;
        if (i<fcollection.Size()) { //Write file/dir
          s=fcollection[i].fullname;
          s=FixStr(s,w.GetWidth()-4); s=(LCopy(s,w.GetWidth()-4,' '));
          if (fcollection[i].isdir) s=("["+s+"]");
        } else { //Fill out blanks
          s=GetSpaces(w.GetWidth());
        }
        w.Write(1,i-topline,s);
      }
    }
    k=GetSomeKey();
    rewrite=true;
    switch (k) {
      default : rewrite=false;
    }
  } while (!KeyIn(k,kEnter, kESC, kF10));
  
}

