/***************************************************************************
       fm_editcon.h  -  An editorclass to fill into a console
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include "fm_editcon.h"
#include "fm_window.h"
#include "fm_helpscreens.h"
#include "fm_conf.h"
#include "sc_shell.h"
#include "fm_entry.h"
#include "sc_file.h"
#include "fm_vfs.h"
#include "fm_menu.h"
#include "fm_viewcon.h" //for global findstring

#define BOTTOMLINE topline+ScreenH()-2
#define LINESONSCREEN ScreenH()-2

//delimiters used by move word forward/backward
const string limitchars=" +-*=()[]{};,.";


cArray<string> clipboard;
cArray<string> linebuffer;

cEditWindow::cEditWindow(const string& fname) {
  error=true;
  if (Configuration.useexternaleditor) {
    DedicatedEdit(filename);
    return;
  }
  
  if (fname=="") {
    ErrorBox("Not a regular file!");
    return;
  }
  if (IsFileBinary(fname)) {
    if (PromptBox("Confirmation","File seems to be binary! Do you really want to edit? (Y/~N)",kEnter,kESC,kY,kN)!=kY) return;
  }

  if (!CopyFile(fname,fname+"~")) {
    if (PromptBox("Confirmation","Could not make backupcopy! Do you really want to edit? (Y/~N)",kEnter,kESC,kY,kN)!=kY) return;
  }

  if (!lines.StartReadFromFile(fname)) {
    ErrorBox("Could not open file!");
    return;
  }
  do {
    finishedreading=!lines.ContinueRead();
  } while (lines.Size()<ScreenH() && (!finishedreading));
  if (lines.TabsPresent()) {
    if (PromptBox("Warning! File contains TAB's",
                  "SC does currently not support these very well. (C~ontinue/[~Cancel])",
                  kEnter,kESC,kC,kO)!=kO) {
      error=true;
      return;
    }
  }
  error=false;
  filename=fname;
  cx=0; cy=0;
  nowblocking=false; blst_x=blst_y=blend_x=blend_y=-1;
  topline=0;
  fromleft=0; fullpath="";
  changedatall=changed=false; indent=true;
  Update();
}

cEditWindow::cEditWindow() {
  lines+="";
  error=false;
  filename="#untitled#";
  cx=0; cy=0;
  topline=0;
  fromleft=0;
  changed=false; indent=true;
  #ifdef _TARGET_LINUX_
    lines.SetFlavor(Unix);
  #else
    lines.SetFlavor(DOS);
  #endif
  finishedreading=true;
  Update();
  nowblocking=false; blst_x=blst_y=blend_x=blend_y=-1;
}

void cEditWindow::Idle() {
  if (!finishedreading) {
    finishedreading=!lines.ContinueRead();
    WriteHeader();
  };
}


cEditWindow::~cEditWindow() {
  if (Configuration.deletebackups) {
    DeleteFile(filename+"~"); //deletebackup
  }
}

//Rewrite a line properly coloring block marking etc.
void cEditWindow::WriteLine(long int i) {
  int ypos=(i-topline)+2;
    if (i>(lines.Size()-1)) {
      Wr(1,ypos,"",cBLACK,cBLUE,ScreenW());
    } else {
      string s;
      if (lines[i].length()>fromleft) {
        s=lines[i].substr(fromleft,ScreenW());
      } else {
        s="";
      }
      int fg=cBLACK; int bg=cWHITE;
      if (i==cy) fg=cRED;
      if (i>=blst_y && i<=blend_y) {
        if (blst_y==blend_y) { //Only some of on line marked
          int xp1=blst_x-fromleft;int xp2=blend_x-fromleft;
          if (xp1<ScreenW()) {
            string s1=s.substr(0,xp1);
            string s2=s.substr(xp1,xp2-xp1);
            string s3;
            if (xp2<=s.length()) s3=s.substr(xp2,9999); else s3="";
            Wr(1,ypos,s1,fg,cWHITE);
            Wr(xp1+1,ypos,s2,fg,cCYAN);
            Wr(xp2+1,ypos,s3,fg,cWHITE,ScreenW());
          } else {
            Wr(1,ypos,s,fg,cWHITE,ScreenW());
          }
        } else
        if (i==blst_y) {
          int xpos=blst_x-fromleft;
          if ((blst_x-fromleft)>0) {
            string s1=s.substr(0,xpos);
            string s2=s.substr(xpos,9999);
            Wr(1,ypos,s1,fg,cWHITE);
            Wr(xpos+1,ypos,s2,fg,cCYAN,ScreenW());
          } else {
            Wr(1,ypos,s,fg,cCYAN,ScreenW());
          }
        } else
        if (i==blend_y) {
          int xpos=blend_x-fromleft;
          if (blend_x<lines[blend_y].length()+1) {
            string s1=s.substr(0,xpos);
            string s2=s.substr(xpos,9999);
            Wr(1,ypos,s1,fg,cCYAN);
            Wr(xpos+1,ypos,s2,fg,cWHITE,ScreenW());
          } else {
            Wr(1,ypos,s,fg,cCYAN,ScreenW());
          }
        } else {
          Wr(1,ypos,s,fg,cCYAN,ScreenW());
          //whole line marked
        }
      } else {
        Wr(1,ypos,s,fg,cWHITE,ScreenW());
      }
    }
}

//Rewrite current line and possibly the liens before and after
void cEditWindow::WriteCurrentLine(bool surround=false) {
  if (surround) {
    if (cy>0) WriteLine(cy-1);
    if (cy<lines.Size()-1) WriteLine(cy+1);
  }
  WriteLine(cy);
  WriteHeader();
  UpdateCursor();
}

void cEditWindow::UpdateCursor() {
  ShowCursor();
  GotoXY(cx-fromleft+1,cy-topline+2); //topcorner=1,1 and text start line 2
}

void cEditWindow::Update() {
  for(int i=topline; i<BOTTOMLINE; i++) WriteLine(i);
  WriteHeader();
  UpdateCursor();
}


void cEditWindow::TreatKey(tKey k) {
  bool refresh=false;
  string saveline=lines[cy]; int savecy=cy; int savecx=cx; int savetopline=topline; int savefromleft=fromleft;
  switch (k) {
    case kBlindDown : {
      if (cy<lines.Size()-1) {
        cy++;
        if (lines[cy].length()<cx) cx=lines[cy].length();
        if (topline<cy-LINESONSCREEN+5) topline++; //5 :daaaahhh!?!?! But it works...
      }
      return;
    }
    case kBlindUp : {
      if (cy>0) cy--;
      if (lines[cy].length()<cx) cx=lines[cy].length();
      if (topline>cy) topline=cy;
      return;
    }
    case kBlindLeft : {
      if (cx>0) {
        cx--;
        if (cx<fromleft) fromleft--;
      } else {
        if (cy>0) {
          cy--;
          TreatKey(kEnd);
        }
      }
      return;
    }
    case kBlindRight : {
      if (cx<lines[cy].length()) {
        cx++;
        if ((cx+1)>(ScreenW()+fromleft)) fromleft++;
      } else {
        if (cy<lines.Size()-1) {
          cy++;
          TreatKey(kHome);
        }
      }
      return;
    }
    case kLeft : TreatKey(kBlindLeft); refresh=false; break;
    case kRight : TreatKey(kBlindRight); refresh=false; break;
    case kUp   : TreatKey(kBlindUp); refresh=false; break;
    case kDown : TreatKey(kBlindDown); refresh=false; break;
    case kPgdn : for (int i=0; i<LINESONSCREEN; i++) TreatKey(kBlindDown); break;
    case kPgup : for (int i=0; i<LINESONSCREEN; i++) TreatKey(kBlindUp); break;
    case kHome : cx=0; fromleft=0;break;
    case kEnd  : {
      while (cx<lines[cy].length()) TreatKey(kBlindRight);
      if (IsKeyPressed()) tKey tk=GetSomeKey(); //flush extra tilde char in linux
      break;
    }

    case kEnter : InsertReturn(); refresh=true; break;
    case kDelete : refresh=DoDelete(); break;
    case kBackspace : refresh=DoBackSpace(); break;

    case kAltC :
    case kCtrlK :
    case kCtrlIns :
    case kCtrlC : nowblocking=false; DoCopy(); break;

    case kAltDel :
    case kShiftDel :
    case kCtrlX : nowblocking=false; refresh=DoCut(); break;

    case kAltIns :
    case kShiftIns :
    case kCtrlV : nowblocking=false; refresh=DoPaste(); break;

    case kCtrlY : refresh=true; CutLine(); break;

    case kCtrlU : refresh=true; PasteLine(); break;

    case kCtrlRight : {
      do {
        TreatKey(kBlindRight);
        if (cy==lines.Size() && cx>=lines[cy].length()) break;
      } while (limitchars.find(CC())==-1);
      do {
        TreatKey(kBlindRight);
        if (cy==lines.Size() && cx>=lines[cy].length()) break;
      } while (limitchars.find(CC())!=-1);
      if (cx==lines[cy].length()) TreatKey(kBlindRight);
      break;
    }

    case kCtrlLeft : {
      do {
        TreatKey(kBlindLeft);
        if (cy==0 && cx==0) break;
      } while (limitchars.find(CC())!=-1);
      do {
        TreatKey(kBlindLeft);
        if (cy==0 && cx==0) break;
      } while (limitchars.find(CC())==-1);
      if (cy!=0 || cx!=0) TreatKey(kBlindRight);
      break;
    }

    case kTab : {
      do {
        InsertChar(' ');
      } while (cx & Configuration.tabsize!=0);
      break;
    }

    

    //Find
    case kF7 :
    case kAltF : {
      refresh=Find();
      break;
    }

    //Find again
    case kAltG : {
      refresh=FindAgain();
      break;
    }

    case kF1 : ShowEditHelp(); refresh=true; break;
    case kCtrlS :
    case kF2 : SaveFile(); refresh=true; break;
    case kAltS : {
      string save=filename;
      filename="#untitled#";
      if (!SaveFile()) filename=save;
      refresh=true; break;
    }
    case kAltM :
    case kF3 : HandleSelection(); break;

    case kAltU :
    case kF4 : {
      blst_x=blst_y=blend_x=blend_y=-1;
      refresh=true;
      nowblocking=false;
      break;
    }


    case kF5 : {
      if (blst_x!=-1) {
        DoCopy(); DoPaste();
        refresh=true;
      }
    }

    case kF6 : {
      if (blst_x!=-1) {
        if (cx!=blend_x && cy!=blend_y) {
          DoCut(); DoPaste();
          refresh=true;
        }
      }
    }


    case kF8 : {
      cMenu m("TextFlavor");
      m+="DOS"; m+="Unix"; m+="Mac";
      int i=m.Run();
      changed=true;
      switch (i) {
        case 1 : lines.SetFlavor(DOS); break;
        case 2 : lines.SetFlavor(Unix); break;
        case 3 : lines.SetFlavor(Mac); break;
        default : ;
      }
      break;
    }

    case kF9 : indent=!indent; refresh=false; break;
    
    default : {
      if (!KeyIn(k,_KEYSET_ALTLETTERS) && !KeyIn(k,_KEYSET_CTRLLETTERS)) {
        InsertChar(Key_LastASCIIChar()); //nouse key
      }
    }

  }
  if (nowblocking) {
    if (cy<=blst_y) {
      if (cy==blst_y && cx>blst_x) {   //Do not extend selection backwards
        blend_x=cx; blend_y=cy;
      }
    } else {
      blend_x=cx; blend_y=cy;
    }
  }
  if (refresh || topline!=savetopline || fromleft!=savefromleft) {
    changedatall=true;
    Update();
  } else {
    if (cy!=savecy) {
      WriteCurrentLine(true);
    } else {
      if (cx!=savecx || saveline!=lines[cy]) WriteCurrentLine();
    }
    changedatall=true;
  }
  WriteHeader();

}



void cEditWindow::InsertChar(char c) {
  if (c=='\0') return;
  lines[cy].insert(cx,1,c);
  cx++;
  if ((cx+1)>(ScreenW()+fromleft)) fromleft++;
  changed=true;
}

void cEditWindow::InsertReturn() {
  changed=true;
  if (cx==0) {
    lines.Insert("",cy);
  } else
  if (cx>lines[cy].length()) {
    lines.Insert("",cy);
  } else {
    string s=lines[cy].substr(cx,9999);
    lines[cy]=lines[cy].substr(0,cx);
    lines.Insert(s,cy+1);
  }
  TreatKey(kBlindDown);
  TreatKey(kHome);
  if (indent) {
    string s=lines[cy-1]; bool empty=true;
    for (int i=0; i<s.length(); i++) if (s[i]!=' ') empty=false;
    if (!empty) {
      int i=0;
      while (s[i++]==' ') InsertChar(' ');
    }
  }
}

bool cEditWindow::DoDelete() {
  changed=true;
  if (cx<lines[cy].length()) {
    lines[cy].replace(cx,1,"");
    return true;
  } else {
    if (cy<lines.Size()-1) {
      string s=(lines[cy]+lines[cy+1]);
      lines[cy]=s;
      lines.Remove(cy+1);
      return true;
    }
  }
  return false;
}

bool cEditWindow::DoBackSpace(){
  changed=true;
  if (cx>0) {
    lines[cy].replace(cx-1,1,"");
    TreatKey(kLeft);
    return true;
  } else {
    if (cy>0) {
      TreatKey(kBlindUp);
      TreatKey(kEnd);
      TreatKey(kDelete);
      //Ahhh, isn't this clever? ;-)
    } //else cx,cy=0,0
  }
  return false;
}

void cEditWindow::DoCopy(bool cut/*=false*/){
  if (blst_x==-1) return;
  clipboard.Clear();
  nowblocking=false;
  if (blst_y==blend_y) { //Selection only within 1 line
    string s2=lines[blst_y].substr(blst_x,blend_x-blst_x); //Grab the selected
    clipboard+=s2; //and copy it to clipboard
    if (cut) {
      string s1=lines[blst_y].substr(0,blst_x);
      string s3=lines[blst_y].substr(blend_x,9999);
      lines[blst_y]=(s1+s3);
    }
  } else {

    for(int i=blst_y; i<=blend_y; i++) {
      if (i==blst_y) {
        string s2=lines[i].substr(blst_x,9999);
        clipboard+=s2;
        if (cut) {
          string s1=lines[i].substr(0,blst_x);
          lines[i]=s1;
        }
      } else
      if (i==blend_y) {
        string s1=lines[i].substr(0,blend_x);
        clipboard+=s1;
        if (cut) {
          string s2=lines[i].substr(blend_x,9999);
          lines[i]=s2;
        }
      } else {
        clipboard+=lines[i]; //only copy completely marked lines
                             //as cutting them would rearrange later lines
      }
    }
  }

}

bool cEditWindow::DoCut(){
  if (blst_x==-1) return false;
  if (cy==blend_y && cx==blend_x) {//are the cursor at the end of the block?
    cx=blst_x; cy=blst_y; //Then put us at the beginning of the block before cutting
  } //else do not move cursor
  changed=true; nowblocking=false;
  DoCopy(true);
  if ((blend_y-blst_y) > 1) { //delete completely marked lines
    for (int i=blst_y+1; i<(blend_y); i++) lines-=i;
  }
  if ((blend_y-blst_y) > 0) { //concenate first and last line
    string s=(lines[blst_y]+lines[blst_y+1]); //cannot use blend_y as it may not be the next
    lines[blst_y]=s;
    lines-=blst_y+1;
  }
  //should delete complete lines and concanated first and last
  nowblocking=false; blst_x=blst_y=blend_x=blend_y=-1;
  return true;
}

bool cEditWindow::DoPaste() {
  if (clipboard.Empty()) return false;
  changed=true; nowblocking=false;
  blst_x=cx; blst_y=cy; //new block will be inserted data
  if (clipboard.Size()==1) {
    string s=(lines[cy].substr(0,cx)+clipboard[0]+lines[cy].substr(cx,9999));
    lines[cy]=s;
    cx=(cx+clipboard[0].length());
    blend_y=cy; blend_x=cx;//blend_x=cx+clipboard[0].length();
    while ((cx+1)>(ScreenW()+fromleft)) {fromleft++;};
    return true;
  }
  string backpack=lines[cy].substr(cx,9999);
  lines[cy]=(lines[cy].substr(0,cx)+clipboard[0]); //first line inserted
  for (int i=1; i<clipboard.Size()-1; i++) { //lines between first and last
    lines.Insert(clipboard[i],cy+i);
    blend_y++;
  }
  string s=clipboard[clipboard.Size()-1]; //Last line
  blend_x=s.length();
  s+=backpack;  //Last line includes backpack
  lines.Insert(s,cy+clipboard.Size()-1);
  blend_y=(cy+clipboard.Size()-1);
  return true;
}

void cEditWindow::CutLine(){
  if (lines.Size()>1) {
    linebuffer.PushStart(lines[cy]);
    lines.Remove(cy);
    if (cy>lines.Size()-1) cy--;
    cx=0;fromleft=0;
    changed=true;
  }
}

void cEditWindow::PasteLine(){
  if (!linebuffer.Empty()) {
    lines.Insert(linebuffer.PopEnd(),cy);
    TreatKey(kDown);
    changed=true;
  };
}


void cEditWindow::WriteHeader() {
  Wr(3,1,GetTitle(),chYELLOW,cBLUE,ScreenW()-2);
  string s="Line "+MkStr(topline+1)+"-";
  if (lines.Size()>BOTTOMLINE) {
    s+=MkStr(BOTTOMLINE); //more lines than on screen
  } else {
    s+=MkStr(lines.Size()); //end of file shown
  }
  //s+="/"+MkStr(lines.Size());
  s+="/"+MkStr(lines.Size());
  s+=(" Cursor:"+MkStr(cx+1)+","+MkStr(cy+1));
  s+="  Type: ";
  switch (lines.GetFlavor()) {
    case DOS : s+="DOS"; break;
    case Unix : s+="Unix"; break;
    case Mac : s+="Mac"; break;
    case Binary : s+="Binary"; break; //should not happen!
  }
  if (changed) s+="  Changed";
  if (indent) s+="  Indent";
  if (nowblocking) s+="  Marking";

  Wr(1,ScreenH(),s,chYELLOW,cBLUE,ScreenW()-1);
  UpdateCursor(); //Always place cursor correctly
}

void cEditWindow::HandleSelection() {
  if (nowblocking) {
    blend_y=cy; blend_x=cx;
    nowblocking=false;
  } else {
    blst_y=blend_y=cy; blst_x=blend_x=cx;
    nowblocking=true;
  }
}



string cEditWindow::GetTitle() {
  return "Edit: "+filename;
}

bool cEditWindow::SaveFile() {
  if (filename=="#untitled#") {
    string s=AddSlash(fullpath);
    if (!InputWindow("Enter filename:",s,50)) return false;
    if (s!="") filename=s; else return false;
  }
  changed=!lines.WriteToFile(filename);
  return !changed;
}

bool cEditWindow::ConfirmExit() {
  if (changed) {
    switch (PromptBox("Confirmation",
                      "Save changes? (~Yes/No/Cancel)",kESC,kEnter,kY,kN,kC)) {
      case kESC :
      case kC : Update(); return false;
      case kN : return true;
      default : return SaveFile();
    }
  } else return true;
}

char cEditWindow::CC() {
  return (lines[cy][cx]);
}

bool cEditWindow::Find() {
  if(InputWindow("Search for:",findstring,40)) {
    return FindAgain();
  }
  return false;
}

bool cEditWindow::FindAgain() {
  for(int i=topline+1; i<lines.Size(); i++) {
    int q=lines[i].find(findstring.c_str());//why doesn't it work with a normal string?
    if(q>-1) {
      topline=i;
      if ((topline+LINESONSCREEN) > lines.Size() ) {
        TreatKey(kEnd);
      }
      return true;
    }
  }
  return false;
}

/*************************************************************************/

bool DedicatedEdit(const string& filename, bool forceexternal) {
  if (Configuration.useexternaleditor || forceexternal) {
    if(Configuration.externaleditor=="") {
      ErrorBox("External editor not set! Using internal");
    } else {
      ExecuteCommand(Configuration.externaleditor+" "+filename, true, true);
      return false;
    }
  }
  cEditWindow ew(filename);
  if (ew.Error()) return false;
  do {
    tKey k;
    do { ew.Idle();} while (!IsKeyPressed());
    k=GetSomeKey();
    if (KeyIn(k,kF10,kESC,kAltX)) {
      if (ew.ConfirmExit()) return ew.ChangedAtAll();
    } else {
      ew.TreatKey(k);
    }
  } while(42); //Know the number? Of course you do...
}


