/***************************************************************************
                          fm_keyboard.cpp  -  description
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : fmpro@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "fm_defs.h"
#include "fm_keyboard.h"
#include "fm_str.h"
#include "fm_window.h"

#include "target.h"
#include "unicon.h"

#ifdef _TARGET_TEXTIO_CURSES_
  #if defined(_TARGET_LINUX_)
    #include <curses.h>
  #elif defined(_TARGET_NETBSD_)
    #include <ncurses.h>
  #else
    #include <curses.h>
  #endif
#endif

char lasttruekey='\0';
int lasttruenumber=-9999;

void InitKeyboard() {
}

void DoneKeyboard() {
}

bool IsKeyPressed() {
  return (ConKbHit()!=0);
}



void TestKeyboard() {
  cWindow w(40,3,"Keyboard test");
  w.Write(1,1,"Press any key, enter exits");
  int c;
  do {
    c=ConGetch();
    string s=(MkStr(c)+",");
    while (IsKeyPressed()) s+=(MkStr(ConGetch())+",");
    s=LCopy(s,38,' ');
    w.Write(1,2,s);
  } while (c!=13);
}


tKey GetSomeKey() {
  int c=0;

  c=ConGetch();
  if (c>31 && c<256) lasttruekey=char(c); else lasttruekey='\0';
  lasttruenumber=-9999;
  switch (c) {
    case 13 : return kEnter;
    case 27 : return kESC;
    case 32 : return kSpace;
    case 263 :
    case 8 : return kBackspace;
    case 9  : return kTab;
    case UCKEY_DEL : return kDelete; case UCKEY_INSERT : return kInsert;
    
    case UCKEY_PPAGE : return kPgup; case UCKEY_NPAGE : return kPgdn;

    case '*' : return kStar; case '+' : return kPlus; case '-' : return kMinus;
    case '/' : return kSlash; case '\\' : return kBackSlash;
    case '.' : return kDot; case ',' : return kComma;
    case 'a' : return kA; case 'A' : return kA; case UCKEY_ALT_A : lasttruekey='A'; return kAltA; case UCKEY_CTRL_A : lasttruekey='A'; return kCtrlA;
    case 'b' : return kB; case 'B' : return kB; case UCKEY_ALT_B : lasttruekey='B'; return kAltB; case UCKEY_CTRL_B : lasttruekey='B'; return kCtrlB;
    case 'c' : return kC; case 'C' : return kC; case UCKEY_ALT_C : lasttruekey='C'; return kAltC; case UCKEY_CTRL_C : lasttruekey='C'; return kCtrlC;
    case 'd' : return kD; case 'D' : return kD; case UCKEY_ALT_D : lasttruekey='D'; return kAltD; case UCKEY_CTRL_D : lasttruekey='D'; return kCtrlD;
    case 'e' : return kE; case 'E' : return kE; case UCKEY_ALT_E : lasttruekey='E'; return kAltE; case UCKEY_CTRL_E : lasttruekey='E'; return kCtrlE;
    case 'f' : return kF; case 'F' : return kF; case UCKEY_ALT_F : lasttruekey='F'; return kAltF; case UCKEY_CTRL_F : lasttruekey='F'; return kCtrlF;
    case 'g' : return kG; case 'G' : return kG; case UCKEY_ALT_G : lasttruekey='G'; return kAltG; case UCKEY_CTRL_G : lasttruekey='G'; return kCtrlG;
    case 'h' : return kH; case 'H' : return kH; case UCKEY_ALT_H : lasttruekey='H'; return kAltH; //case UCKEY_CTRL_H : lasttruekey='H'; return kCtrlH;
    case 'i' : return kI; case 'I' : return kI; case UCKEY_ALT_I : lasttruekey='I'; return kAltI; //case UCKEY_CTRL_I : lasttruekey='I'; return kCtrlI;
    case 'j' : return kJ; case 'J' : return kJ; case UCKEY_ALT_J : lasttruekey='J'; return kAltJ; case UCKEY_CTRL_J : lasttruekey='J'; return kCtrlJ;
    case 'k' : return kK; case 'K' : return kK; case UCKEY_ALT_K : lasttruekey='K'; return kAltK; case UCKEY_CTRL_K : lasttruekey='K'; return kCtrlK;
    case 'l' : return kL; case 'L' : return kL; case UCKEY_ALT_L : lasttruekey='L'; return kAltL; case UCKEY_CTRL_L : lasttruekey='L'; return kCtrlL;
    case 'm' : return kM; case 'M' : return kM; case UCKEY_ALT_M : lasttruekey='M'; return kAltM; //case UCKEY_CTRL_M : lasttruekey='M'; return kCtrlM;
    case 'n' : return kN; case 'N' : return kN; case UCKEY_ALT_N : lasttruekey='N'; return kAltN; case UCKEY_CTRL_N : lasttruekey='N'; return kCtrlN;
    case 'o' : return kO; case 'O' : return kO; case UCKEY_ALT_O : lasttruekey='O'; return kAltO; case UCKEY_CTRL_O : lasttruekey='O'; return kCtrlO;
    case 'p' : return kP; case 'P' : return kP; case UCKEY_ALT_P : lasttruekey='P'; return kAltP; case UCKEY_CTRL_P : lasttruekey='P'; return kCtrlP;
    case 'q' : return kQ; case 'Q' : return kQ; case UCKEY_ALT_Q : lasttruekey='Q'; return kAltQ; case UCKEY_CTRL_Q : lasttruekey='Q'; return kCtrlQ;
    case 'r' : return kR; case 'R' : return kR; case UCKEY_ALT_R : lasttruekey='R'; return kAltR; case UCKEY_CTRL_R : lasttruekey='R'; return kCtrlR;
    case 's' : return kS; case 'S' : return kS; case UCKEY_ALT_S : lasttruekey='S'; return kAltS; case UCKEY_CTRL_S : lasttruekey='S'; return kCtrlS;
    case 't' : return kT; case 'T' : return kT; case UCKEY_ALT_T : lasttruekey='T'; return kAltT; case UCKEY_CTRL_T : lasttruekey='T'; return kCtrlT;
    case 'u' : return kU; case 'U' : return kU; case UCKEY_ALT_U : lasttruekey='U'; return kAltU; case UCKEY_CTRL_U : lasttruekey='U'; return kCtrlU;
    case 'v' : return kV; case 'V' : return kV; case UCKEY_ALT_V : lasttruekey='v'; return kAltV; case UCKEY_CTRL_V : lasttruekey='V'; return kCtrlV;
    case 'w' : return kW; case 'W' : return kW; case UCKEY_ALT_W : lasttruekey='W'; return kAltW; case UCKEY_CTRL_W : lasttruekey='W'; return kCtrlW;
    case 'x' : return kX; case 'X' : return kX; case UCKEY_ALT_X : lasttruekey='X'; return kAltX; case UCKEY_CTRL_X : lasttruekey='X'; return kCtrlX;
    case 'y' : return kY; case 'Y' : return kY; case UCKEY_ALT_Y : lasttruekey='Y'; return kAltY; case UCKEY_CTRL_Y : lasttruekey='Y'; return kCtrlY;
    case 'z' : return kZ; case 'Z' : return kZ; case UCKEY_ALT_Z : lasttruekey='Z'; return kAltZ; case UCKEY_CTRL_Z : lasttruekey='Z'; return kCtrlZ;

    case '1' : lasttruenumber=1; return k1; case UCKEY_ALT_1 : lasttruenumber=1; lasttruekey='1'; return kAlt1;
    case '2' : lasttruenumber=2; return k2; case UCKEY_ALT_2 : lasttruenumber=2; lasttruekey='2'; return kAlt2;
    case '3' : lasttruenumber=3; return k3; case UCKEY_ALT_3 : lasttruenumber=3; lasttruekey='3'; return kAlt3;
    case '4' : lasttruenumber=4; return k4; case UCKEY_ALT_4 : lasttruenumber=4; lasttruekey='4'; return kAlt4;
    case '5' : lasttruenumber=5; return k5; case UCKEY_ALT_5 : lasttruenumber=5; lasttruekey='5'; return kAlt5;
    case '6' : lasttruenumber=6; return k6; case UCKEY_ALT_6 : lasttruenumber=6; lasttruekey='6'; return kAlt6;
    case '7' : lasttruenumber=7; return k7; case UCKEY_ALT_7 : lasttruenumber=7; lasttruekey='7'; return kAlt7;
    case '8' : lasttruenumber=8; return k8; case UCKEY_ALT_8 : lasttruenumber=8; lasttruekey='8'; return kAlt8;
    case '9' : lasttruenumber=9; return k9; case UCKEY_ALT_9 : lasttruenumber=9; lasttruekey='9'; return kAlt9;
    case '0' : lasttruenumber=0; return k0; case UCKEY_ALT_0 : lasttruenumber=0; lasttruekey='0'; return kAlt0;
    

    
    case UCKEY_F1 : return kF1; case UCKEY_F6 : return kF6;
    case UCKEY_F2 : return kF2; case UCKEY_F7 : return kF7;
    case UCKEY_F3 : return kF3; case UCKEY_F8 : return kF8;
    case UCKEY_F4 : return kF4; case UCKEY_F9 : return kF9;
    case UCKEY_F5 : return kF5;
//    case UCKEY_unicon_F0 :
    case UCKEY_F10 : return kF10;

    case UCKEY_SH_F1 : lasttruenumber=1; return kShiftF1;
    case UCKEY_SH_F6 : lasttruenumber=6; return kShiftF6;
    case UCKEY_SH_F2 : lasttruenumber=2; return kShiftF2;
    case UCKEY_SH_F7 : lasttruenumber=7; return kShiftF7;
    case UCKEY_SH_F3 : lasttruenumber=3; return kShiftF3;
    case UCKEY_SH_F8 : lasttruenumber=8; return kShiftF8;
    case UCKEY_SH_F4 : lasttruenumber=4; return kShiftF4;
    case UCKEY_SH_F9 : lasttruenumber=9; return kShiftF9;
    case UCKEY_SH_F5 : lasttruenumber=5; return kShiftF5;
    case UCKEY_SH_F10 : lasttruenumber=10; return kShiftF10;

    case UCKEY_LEFT : return kLeft; case UCKEY_RIGHT : return kRight;
    case UCKEY_UP : return kUp; case UCKEY_DOWN : return kDown;
    case UCKEY_END : return kEnd; case UCKEY_HOME : return kHome;


    #ifdef _TARGET_DJGPP_
      case 1038 : return kAltBackspace;
      case 1139: return kCtrlLeft; case 1140: return kCtrlRight;
      case 1179: return kAltLeft;  case 1181: return kAltRight;
      case 1170: return kAltIns;   case 1171: return kAltDel;
      case 1186: return kCtrlIns;  case 1187: return kCtrlDel;
    #endif

    #ifdef _TARGET_LINUX_
/*      case 1287 : return kAltBackspace;
      case 1289 : return kShiftF1;
      case 1290 : return kShiftF2;
      case 1291 : return kShiftF3;
      case 1292 : return kShiftF4;
      case 1293 : return kShiftF5;
      case 1294 : return kShiftF6;
      case 1295 : return kShiftF7;
      case 1296 : return kShiftF8;
      case 1297 : return kShiftF9;
      case 1298 : return kShiftF10;
*/

//      case 1289 : return kShiftF11;
//      case 1289 : return kShiftF12;
      
      //rxvt keys
      case 385 : return kEnd;
      case 362 : return kHome;
      case 127 : return kBackspace;

      //Hacks to make Linux Console keyboard right
      //I don't know if this works other places than here, but maybe...
      case 1115 : {
        int c1=ConGetch();
        switch (c1) {
          case 50 : {
            int c2=ConGetch(); 
            int c3=ConGetch(); //allways 126?
            switch (c2) {
              case 49 : return kF10;
              case 51 : return kF11;
              case 52 : return kF12;
              case 53 : lasttruenumber=1; return kShiftF1;
              case 54 : lasttruenumber=2; return kShiftF2;
              case 56 : lasttruenumber=3; return kShiftF3;
              case 57 : lasttruenumber=4; return kShiftF4;
              default : return kDummyKey;
            }
          }
          case 51 : {
            int c2=ConGetch(); 
            int c3=ConGetch(); //allways 126?
            switch (c2) {
              case 49 : lasttruenumber=5; return kShiftF5;
              default : return kDummyKey;
            }
          }
          case 70 : //Happens in some xterms
          case 52 : return kEnd;
          case 72 : return kHome;
          default : return kDummyKey;
        }
      }
    #endif
    
    default : return kDummyKey;
  }

}


char Key_LastASCIIChar() {
  return lasttruekey;
}

int Key_LastNumber() {
  return lasttruenumber;
}

tKey GetThatKey(tKey k0, tKey k1/*=kDummyKey*/, tKey k2/*=kDummyKey*/, tKey k3/*=kDummyKey*/,
                           tKey k4/*=kDummyKey*/, tKey k5/*=kDummyKey*/, tKey k6/*=kDummyKey*/,
                           tKey k7/*=kDummyKey*/, tKey k8/*=kDummyKey*/, tKey k9/*=kDummyKey*/,
                           tKey k10/*=kDummyKey*/, tKey k11/*=kDummyKey*/, tKey k12/*=kDummyKey*/) {

  int c=kDummyKey;
  do {
    tKey k=GetSomeKey();
    if (k==k0) return k;
    if (k==k1 && k1!=kDummyKey) return k; if (k==k2 && k2!=kDummyKey) return k;
    if (k==k3 && k3!=kDummyKey) return k; if (k==k4 && k4!=kDummyKey) return k;
    if (k==k5 && k5!=kDummyKey) return k; if (k==k6 && k6!=kDummyKey) return k;
    if (k==k7 && k7!=kDummyKey) return k; if (k==k8 && k8!=kDummyKey) return k;
    if (k==k9 && k9!=kDummyKey) return k; if (k==k10 && k10!=kDummyKey) return k;
  } while (c==kDummyKey);
  return kDummyKey;
}

bool KeyIn(tKey k, tKey k1,tKey k2,   tKey k3/*=kDummyKey*/,  tKey k4/*=kDummyKey*/,
                   tKey k5/*=kDummyKey*/, tKey k6/*=kDummyKey*/,  tKey k7/*=kDummyKey*/,
                   tKey k8/*=kDummyKey*/, tKey k9/*=kDummyKey*/,  tKey k10/*=kDummyKey*/,
                   tKey k11/*=kDummyKey*/,tKey k12/*=kDummyKey*/, tKey k13/*=kDummyKey*/,
                   tKey k14/*=kDummyKey*/, tKey k15/*=kDummyKey*/,  tKey k16/*=kDummyKey*/,
                   tKey k17/*=kDummyKey*/,tKey k18/*=kDummyKey*/,  tKey k19/*=kDummyKey*/,
                   tKey k20/*=kDummyKey*/, tKey k21/*=kDummyKey*/,  tKey k22/*=kDummyKey*/,
                   tKey k23/*=kDummyKey*/,tKey k24/*=kDummyKey*/, tKey k25/*=kDummyKey*/,
                   tKey k26/*=kDummyKey*/,tKey k27/*=kDummyKey*/, tKey k28/*=kDummyKey*/) {
  if (k==k1) return true; if (k==k2) return true;
  if (k==k3 && k3!=kDummyKey) return true;
  if (k==k4 && k4!=kDummyKey) return true;
  if (k==k5 && k5!=kDummyKey) return true;
  if (k==k6 && k6!=kDummyKey) return true;
  if (k==k7 && k7!=kDummyKey) return true;
  if (k==k8 && k8!=kDummyKey) return true;
  if (k==k9 && k9!=kDummyKey) return true;
  if (k==k10 && k10!=kDummyKey) return true;
  if (k==k11 && k11!=kDummyKey) return true;
  if (k==k12 && k12!=kDummyKey) return true;
  if (k==k13 && k13!=kDummyKey) return true;
  if (k==k14 && k14!=kDummyKey) return true;
  if (k==k15 && k15!=kDummyKey) return true;
  if (k==k16 && k16!=kDummyKey) return true;
  if (k==k17 && k17!=kDummyKey) return true;
  if (k==k18 && k18!=kDummyKey) return true;
  if (k==k19 && k19!=kDummyKey) return true;
  if (k==k20 && k20!=kDummyKey) return true;
  if (k==k21 && k21!=kDummyKey) return true;
  if (k==k22 && k22!=kDummyKey) return true;
  if (k==k23 && k23!=kDummyKey) return true;
  if (k==k24 && k24!=kDummyKey) return true;
  if (k==k25 && k25!=kDummyKey) return true;
  if (k==k26 && k26!=kDummyKey) return true;
  if (k==k27 && k27!=kDummyKey) return true;
  if (k==k28 && k28!=kDummyKey) return true;

  return false;
}

void WaitKey() {
  GetSomeKey();
};
