#include "fm_scd.h"
#include "target.h"
#include "fm_vfs.h"
#include "fm_fileinfo.h"
#include "fm_window.h"
#include "fm_entry.h"

class cSCDir {
public:
  cSCDir(const string& dirname) {
    name=dirname;
    link=NULL;
  }
  string name;
  cSCDir* link;
};

bool prompting;

cSCDir *cachetop=NULL;
cSCDir *cacheend=NULL;

void PushDir(const string& name) {
  cSCDir *newdir=new cSCDir(name);
  if (cachetop==NULL) {
    cachetop=cacheend=newdir;
  } else {
    cacheend->link=newdir;
    cacheend=newdir;
  }
}

string PopDir() {
  if (cachetop==NULL) return "";
  string s=cachetop->name;
  cSCDir *temp=cachetop;
  cachetop=cachetop->link;
  delete temp;
  return s;
}

void DestroyCache() {
  cSCDir *temp=cachetop->link;
  while (temp!=NULL) {
    delete cachetop;
    cachetop=temp;
    temp->link;
  }
  cachetop=cacheend=NULL;
}

bool ConfirmFound(string& dirname) {
  cWindow w(52,2,"Found as:");
  w.ChDnleft("~T~A~B keep searching");
  w.ChDnright("~E~N~T~E~R accept");
  w.RedrawBox();
  w.Write(1,1,dirname);
  switch (GetThatKey(kEnter,kESC,kTab)) {
    case kEnter : return true;
    case kTab : return false;
    default : dirname="***breaked***"; return true;
  }
  return false;
}

string SearchDir(const string& dirname,
                 const string& searchfor,
                 bool findfile,
                 bool confirm) {
  ShowMessage(FixStr(dirname,ScreenW()-10),"Searching...",cWHITE);
  string path=(AddSlash(dirname)+searchfor);
  if (findfile) {
    if (DoesFileExist(path)) {
      if (prompting) {
        if (ConfirmFound(path)) return (dirname);
      } else return (dirname);
    }
  } else {
    if(DoesDirExist(path)) {
      if (prompting) {
        if (ConfirmFound(path)) return (path);
      } else return (path);
    }

  }
  if (IsKeyPressed()) return "***breaked***";
  tDirContainer tdc;
  if (ScanForDirs(tdc,dirname)) {
    //Width first
    for(int i=0; i<tdc.Size(); i++) PushDir(tdc[i].fullname);
    ShowRotator();
  }
  return "";
}

string StartSearch(const string& searchfor, bool findfile, bool confirm) {
  #ifdef _TARGETFLAVOR_UNIX_
    string s=SearchDir("/",searchfor,findfile,confirm);
  #endif
  #ifdef _TARGETFLAVOR_DOS_
    string s=CurrentDir();
    s=s.substr(0,3);
    s=SearchDir(s,searchfor,findfile,confirm);
  #endif
  if (s=="") {
    s=PopDir();
    string result="";
    while (s!="" && result=="") {
      result=SearchDir(s,searchfor,findfile,confirm);
      s=PopDir();
    }
    return result;
  } else return s;
  DestroyCache();
}

string SCD_GetDir() {
  static string lastsearch="";
  if (InputWindow("Dir to search for (\\ for prompt):",lastsearch,50)) {
    prompting=StripChar(lastsearch,'\\');
    ShowMessage("Searching...");
    string result=StartSearch(lastsearch,false,false);
    if (result=="***breaked***") return ""; else return result;
  }
  return "";
}

string SCD_FindFile(string& filename) {
  static string lastsearch="";
  if (InputWindow("File to search for (\\ for prompt):",lastsearch,50)) {
    prompting=StripChar(lastsearch,'\\');
    ShowMessage("Searching...");
    string result=StartSearch(lastsearch,true,false);
    if (result=="***breaked***") {
      filename="";
      return "";
    } else {
      filename=lastsearch;
      return result;
    }
  }
  return "";
}
