/***************************************************************************
                 fm_sysinfo.cpp  -  show some systeminfo
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string>
#include <stdlib.h>
#include "target.h"
#ifndef _TARGET_MINGW_
  #include <sys/vfs.h>
#endif
#include "fm_debug.h"
#include "fm_defs.h"
#include "fm_str.h"
#include "fm_sysinfo.h"
#include "fm_window.h"
#include "fm_keyboard.h"
#include "fm_str.h"
#include "fm_textfile.h"
#include "fm_vfs.h"
#include "compiled.h"
#include "README.h"
#include "ChangeLog.h"
#include "sc_update.h"

string ProcessorID() {
  #ifdef _TARGET_LINUX_
    cTextFile tf;
    if (!tf.ReadWholeFile("/proc/cpuinfo")) {
      return "Unknown";
    }
    int i=tf.FindString("model name");
    string s=tf[i];
    i=s.find(":");
    if (i!=-1) {
      return StripSurroundSpace(s.substr(i+1,999));
    } else return "Unknown";
  #endif
  return "unknown";
}

string ProcessorSpeed() {
  #ifdef _TARGET_LINUX_
    cTextFile tf;
    if (!tf.ReadWholeFile("/proc/cpuinfo")) {
      return "Unknown";
    }
    int i=tf.FindString("MHz");
    string s=tf[i];
    i=s.find(":");
    if (i!=-1) {
      return (StripSurroundSpace(s.substr(i+1,999)+" MHz"));
    } else return "Unknown";
  #endif
  return "unknown";
}

string MemInstalled() {
  #ifdef _TARGET_LINUX_
    cTextFile tf;
    if (!tf.ReadWholeFile("/proc/meminfo")) {
      return "Unknown";
    }
    int i=tf.FindString("MemTotal");
    string s=tf[i];
    i=s.find(":");
    if (i!=-1) {
      return StripSurroundSpace(s.substr(i+1,99));
    } else return "Unknown";
  #endif
  return "unknown";
}



void ShowREADME() {
  cScrollingInfoWindow w("README");
  for (int i=0; i<README_numberoflines; i++) {
    w.AddLine(README[i]);
  }
  tKey k=w.Show();
}

void ShowChangeLog() {
  cScrollingInfoWindow w("ChangeLog");
  for (int i=0; i<ChangeLog_numberoflines; i++) {
    w.AddLine(ChangeLog[i]);
  }
  tKey k=w.Show();
}



void ShowSystemInfo() {
  cInfoWindow w("SystemInfo");
  string s="Sunshine Commander version "; s+=_VERSION;
  w.AddLine(s,chYELLOW,_centered);
  s="CopyRight 1999-2000 Morten Poulsen";
  w.AddLine(s,cBLACK,_centered);
  w.AddLine();
  
  s="Built for  : "; s+=_TARGET_DESCRIPTION_;
  w.AddLine(s,cBLACK);
  s="LastUpdate : "; s+=_COMPILETIME;
  w.AddLine(s,cBLACK);
  s="Debugging  : "; if (IsDebuggingEnabled()) s+="Enabled"; else s+="Disabled";
  w.AddLine(s,cBLACK);

  w.AddLine();

  w.AddLine("HomePage: http://sc.poulsen.org");
  w.AddLine("Email   : sc@poulsen.org");
  w.AddLine();

  #ifdef _TARGETFLAVOR_UNIX_
    s="Terminal Type : ";
    if (!getenv("TERM")) s+="Not set"; else s+=getenv("TERM");
    w+=s;
    s="Shell         : ";
    if (!getenv("SHELL")) s+="Not set"; else s+=getenv("SHELL");
    w+=s;
  #endif
  #ifdef _TARGET_DJGPP_
    s="ComSpec  : ";
    if (!getenv("COMSPEC")) s+="Not set"; else s+=getenv("COMSPEC");
    w+=s;
  #endif

  #ifdef _TARGET_LINUX_
    w+="";
    w+=("Processor      : "+ProcessorID());
    w+=("ProcessorSpeed : "+ProcessorSpeed());
    w+=("Memory found   : "+MemInstalled());
  #endif

  w+="";
  w.AddLine("Press 1 to view README file   ",chYELLOW,_centered);
  w.AddLine("Press 2 to view Changelog file",chYELLOW,_centered);
  w.AddLine("Press 3 to check for updates  ",chYELLOW, _centered);
    
  tKey k=w.Show();
  switch (k) {
    case kD : ShowDebugInfo(); break;
    case kK : TestKeyboard(); break;
    case k1 : ShowREADME(); break;
    case k2 : ShowChangeLog(); break;
    case k3 : CheckForUpdates(); break;
  }
}

void ShowDriveInfo() {

#ifdef _TARGETFLAVOR_DOS_
  const int _spacefield=16;
  cInfoWindow w("Drive Info",cWHITE);
  string s="Drive FSys";

  s+=LeadingSpace("Total Space",_spacefield);
  s+=LeadingSpace("Free Space",_spacefield);
  s+="  % free";
  w.AddLine(s,chYELLOW);
  tFilesystems fs;
  if (GetFullFilesystemInfo(fs)) {
    for (int i=1; i<fs.Size(); i++) { //1= skip floppydrive
      s=fs[i].device+"     ";
      s+=fs[i].filesystem;
      s+=LeadingSpace(MkStr(fs[i].total,true),_spacefield);
      s+=LeadingSpace(MkStr(fs[i].avail,true),_spacefield);
      float f=(fs[i].avail*1.0)/fs[i].total;
      s+=LeadingSpace((MkStr((int)(f*100))),7);
      s+="%";
      w+=s;
    }
  } else {
    w+="Error getting filesystem info";
  }
#endif


#ifdef _TARGETFLAVOR_UNIX_
  cInfoWindow w("FileSystem Info",cWHITE);
  const int _devsize=32;
  string s=LCopy("Device",_devsize,' ');
  s+=LCopy("MountPoint",_devsize,' ');
  s+="FSys ";
  w.AddLine(s,chYELLOW);
  tFilesystems fs;
  if (GetFullFilesystemInfo(fs)) {
    for (int i=0; i<fs.Size(); i++) {
      s=LCopy(fs[i].device,_devsize,' ');
      s+=LCopy(fs[i].mountpoint,_devsize,' ');
      s+=LCopy(fs[i].filesystem,12,' ');
      w+=s;
      if (fs[i].total!=0) {
        s="  ";
        s+=" Total: ";
        s+=MkStr(fs[i].total,true);
        s+=" bytes  Free:";
        s+=MkStr(fs[i].avail,true);
        double f=(fs[i].avail*1.0)/fs[i].total;
        s+=" (";
        s+=(MkStr((int)(f*100)));
        s+="%)";
        w.AddLine(s,cYELLOW);

      }
    }
  } else {
    w+="Error getting filesystem info";
  }
#endif //_TARGETFLAVOR_UNIX_
  w.Show();
}


