#ifndef _FM_TREE
#define _FM_TREE

#include <string>
#include "fm_vfs.h"
#include "fm_conio.h"
#include "array.h"
#include "target.h"
#include "fm_keyboard.h"

class cTreeNode {
  public:
    cTreeNode(const cDirEntry&, cTreeNode*);
    cTreeNode(const string& s,cTreeNode* brother, cTreeNode* mother=NULL);
    ~cTreeNode();
    bool Expand(bool=false);
    void Contract();
    string fullname,name;
    bool exp,current;                       //already expanded
    bool packed;
    bool islink;
    bool ismount;
    string linkto;
    cTreeNode *samef, *children, *parent; //same level forward,
                                          //subdirs, superdir
};


struct sShowNode {
  string showline;
  string fullpath;
  bool current;
  cTreeNode *link;
  int realnamestart;
};


class cFmTree {
  public:
    cFmTree();
    ~cFmTree();
    string GetPath(const string&, const string&, const string& defaultdest="",bool=false, bool=false);
  private:
    cArray<sShowNode> showlines;
    int cur, topline;
    bool allowgotodest,gotodest;
    string search;
    sBox box;
    cTreeNode* top;
    string show_dir, show_bottom, show_line, title;

    #ifdef _TARGETFLAVOR_DOS_
      cTreeNode* a_drive;
    #endif
    #ifdef _TARGETFLAVOR_UNIX_
      cTreeNode* mounts;
    #endif
    

    void MakeShowTree(cTreeNode*, const string&, int, int&);

    void CalcTopline();
    int Update();
    void UpdateView();
    cTreeNode* FillInDir(const string&,bool=true);
    int FindDir(cTreeNode*);
    bool TreatKey(tKey&);
    void _TreatKey(tKey ink) {tKey k=ink; TreatKey(k);};
    void ClearCurrent(cTreeNode*);
    bool SearchDirChar();
    void Rescan();
    bool HandleDirSearch(tKey);
    #ifdef _TARGETFLAVOR_DOS_
      void CheckDrives();
    #endif
    #ifdef _TARGETFLAVOR_UNIX_
      void CheckMounts();
    #endif
};

extern cFmTree* FmTree;
extern string Tree_GotoDestination;

#endif //_FM_TREE

