/***************************************************************************
                 fm_window.cpp  -  Basic windowing system
                             -------------------
    begin                : Sun Dec 19 1999
    copyright            : (C) 1999 by MoP
    email                : fmpro@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <errno.h>
#include "fm_window.h"
#include "fm_str.h"
#include "fm_conf.h"

cWindow::cWindow(int sizex, int sizey, const string& title/*=""*/, int color/*=_WINDOW_DEFAULTBG*/ ) {
  width=sizex;
  height=sizey;
  bgcolor=color;
  sUpleft=title;
  box=GetBoxChars();
  RedrawBox();
  HideCursor();
}

cWindow::~cWindow() {

}

void cWindow::FastWrite(int x, int y, const string& s, int fgcol/*=cBLACK*/, int bgcol/*=-1*/) {
  lastx=x; lasty=y; lastlen=s.length();
  if (bgcol==-1) bgcol=bgcolor;
  /*if (x==_centered) {
    x=(width-s.length())/2;
  }
  if (x==_rightaligned) {
    x=(width-s.length());
  }*/
  
  //WrHl(x+posx,y+posy,s,fgcol,bgcol);
  Wr(x+posx,y+posy,s,fgcol,bgcol);
}


void cWindow::Write(int x, int y, const string& s, int fgcol/*=cBLACK*/, int bgcol/*=-1*/) {
  lastx=x; lasty=y;lastlen=s.length();
  if (bgcol==-1) bgcol=bgcolor;
  if (x==_centered) {
    x=(width-s.length())/2;
  }
  if (x==_rightaligned) {
    x=(width-s.length());
  }  
  WrHl(x+posx,y+posy,s,fgcol,bgcol);
}

void cWindow::JustWrite(const string& s, int fgcol/*=cBLACK*/, int bgcol/*=-1*/) {
  Write(lastx,lasty+1,s,fgcol,bgcol);
   
}

void cWindow::WriteOn(const string& s, int fgcol/*=cBLACK*/, int bgcol/*=-1*/) {
  Write(lastx+lastlen,lasty,s,fgcol,bgcol);
  lastx-=s.size();
  
}

void cWindow::DrawHeader() {
  string header="", footer="";
  if (sUpleft.length()>0) {
    header=box.upleft+box.titleleft+sUpleft+box.titleright;
  } else {
    header=box.upleft;
  }

  if (sUpright.length()>0) {
    header+=GetHLine(width-header.length()-sUpright.length()-3);
    header+=box.titleleft+sUpright+box.titleright+box.upright;
  } else {
    header+=GetHLine(width-header.length()-1);
    header+=box.upright;
  }

  if (sDnleft.length()>0) {
    footer=box.dnleft+box.titleleft+sDnleft+box.titleright;
  } else {
    footer=box.dnleft;
  }

  if (sDnright.length()>0) {
    footer+=GetHLine(width-footer.length()-sDnright.length()-3);
    footer+=box.titleleft+sDnright+box.titleright+box.dnright;
  } else {
    footer+=GetHLine(width-footer.length()-1);
    footer+=box.dnright;
  }
  WrHl(posx,posy,header,cBLACK,bgcolor);
  WrHl(posx,posy+height,footer,cBLACK,bgcolor);
}

void cWindow::RedrawBox() {
  ReCalcBox();
  DrawHeader();
  string fill=box.vline+GetSpaces(width-2)+box.vline;
  for(int i=1; i<height; i++) {
    Wr(posx,posy+i,fill,cBLACK,bgcolor);
  }
}

void cWindow::ReCalcBox() {
  posx=(ScreenW()-width) / 2;
  posx++;
  posy=(ScreenH()-height) / 2;
  lastx=0; lasty=0;
}

/*************************************************************************/


tKey cInfoWindow::Show(bool waitforkey/*=true*/) {
  if (lines.size()>0) {
    if (width>ScreenW()) width=ScreenW();
    cWindow w(width,lines.size()+1,title,bg);
    if (dnright!="") w.ChDnright(dnright);
    if (dnleft!="") w.ChDnleft(dnleft);
    if (upright!="") w.ChUpright(upright);
    w.RedrawBox();
    for (int i=0; i<lines.size(); i++) {
      w.Write(x[i],i+1,lines[i],colors[i]);
    }
    if (waitforkey) return GetSomeKey();
  }
  return kDummyKey;
}

/*************************************************************************/

void cScrollingInfoWindow::Redraw() {
  int i=top;
  while (i-top<height) {
    string s=LCopy(lines[i],width,' ');
    w->FastWrite(x[i],i-top+1,s,colors[i]);
    i++;
  }
}

tKey cScrollingInfoWindow::Show() {
  tKey k=kDummyKey;
  if (lines.size()>0) {
    if (width>ScreenW()-4) width=ScreenW()-4;
    if (lines.size()>ScreenH()-4) height=ScreenH()-4; else height=lines.size();
    
    w=new cWindow(width+2,height+2,title,bg);
    if (dnright!="") w->ChDnright(dnright);
    if (dnleft!="") w->ChDnleft(dnleft);
    if (upright!="") w->ChUpright(upright);
    w->RedrawBox();
    bool redraw=true, exit=false;
    top=0;
    do {
      if (redraw) Redraw();
      redraw=true;
      k=GetSomeKey();
      switch (k) {
        case kDown : {
          if (lines.size()>height) {
            if (top+height<lines.size()) top++; else redraw=false;
          }
          break;
        }
        case kUp : {
          if (lines.size()>height) {
            if (top>0) top--; else redraw=true;
          }
          break;
        }
        case kPgdn : {
          if (lines.size()>height) {
            if (top+height<lines.size()) top+=height; else redraw=false;
            if (top+height>lines.size()) top=lines.size()-height;
          }
          break;
        }
        case kPgup : {
          if (lines.size()>height) {
            if (top>0) top-=height; else redraw=false;
            if (top<0) top=0;
          }
          break;
        }
        case kHome : {
          if (lines.size()>height) {
            if (top>0) top=0; else redraw=false;
          }
          break;
        }
        case kEnd : {
          if (lines.size()>height) {
            top=lines.size()-height;
          }
          break;
        }
        default : redraw=false; exit=true;
      }
    } while (!exit);
  }
  delete w;
  return k;
}


/*************************************************************************/
void ShowMessage(const string& is,const string& title/*=""*/, int bg/*=_WINDOW_DEFAULTBG*//*,int width=-1*/) {
  //if (width==-1) int width=s.length()+2;
  //cWindow w(width,2,"",bg);
  //w.Write(1,1,s);
  string s=is;
  cInfoWindow iw(title,bg);
  int i=s.find("^");
  do {
    if (i!=-1) {
      iw.AddLine(s.substr(0,i),cBLACK,_centered);
      s=s.substr(i+1,999);
    }
    i=s.find("^");
  } while (i!=-1);
  iw.AddLine(s,cBLACK,_centered);
  iw.Show(false);
}

void MessageBox(const string& s,const string& title/*=""*/, int bg/*=_WINDOW_DEFAULTBG*//*,int width=-1*/) {
  ShowMessage(s,title,bg);
  WaitKey();  
}

void ErrorBox(const string& s) {
  //cWindow w(s.length()+2,2,"Error",cRED);
  //w.Write(1,1,s);
  ShowMessage(s,"Error",cRED);
  WaitKey();
}

tKey PromptBox(const string& title, const string& message,
  tKey k1, tKey k2/*=kDummyKey*/, tKey k3/*=kDummyKey*/, tKey k4/*=kDummyKey*/,
  tKey k5/*=kDummyKey*/, tKey k6/*=kDummyKey*/,tKey k7, tKey k8) {
  //cWindow w(message.length()+2,2,title,cGREEN);
  //w.Write(1,1,message);
  ShowMessage(message,title,cGREEN);
  return GetThatKey(k1,k2,k3,k4,k5,k6,k7,k8);
}


void ShowRotator() {
  Wr(ScreenW(),1,GetRotator(),cWHITE,cBLUE,0);
}


string GetProgressBar(long int part, long int total,int length) {
  string s;
  if (total<1) return "";
  double p=part; double t=total;
  if (p>t) p=t;
  if (p<0) p=0;
  length-=5; //percentsize
  s=LCopy((GetPercent(part,total)+" "),4,' ');
  int filled=(int)((p/t)*length);
  if (Configuration.usesimplechars) {
    s=(s+LCopy("",filled,'#')+GetSpaces(length-filled));
  } else {
    s=(s+LCopy("",filled,'')+LCopy("",length-filled,''));
  }
  return s;
}

void ShowProgressWindow(long int part, long int total, int length/*=40*/, const string& title/*=""*/) {
  cWindow w(length+2,2,title);
  w.Write(1,1,GetProgressBar(part,total,length),chYELLOW);
}


void ShowErrorMessage() {
   switch (errno) {
     case EDOM   : ErrorBox("EDOM - Input to function out of range"); break;
     case ERANGE : ErrorBox("ERANGE - Output of function out of range"); break;
     case E2BIG  : ErrorBox("E2BIG - Argument list too long"); break;
     case EACCES : ErrorBox("EACCES - Permission denied"); break;
     case EAGAIN : ErrorBox("EAGAIN - Resource temporarily unavailable"); break;
     case EBADF  : ErrorBox("EBADF - Bad file descriptor"); break;
     case EBUSY  : ErrorBox("EBUSY - Resource busy"); break;
     case EEXIST : ErrorBox("EEXIST - File exists"); break;
     case ECHILD : ErrorBox("ECHILD - No child processes"); break;
     case EDEADLK: ErrorBox("EDEADLK - Resource deadlock avoided"); break;
     case EFAULT : ErrorBox("EFAULT - Bad address"); break;
     case EFBIG  : ErrorBox("EFBIG - File too large"); break;
     case EINTR  : ErrorBox("EINTR - Interrupted system call"); break;
     case EINVAL : ErrorBox("EINVAL - Invalid argument"); break;
     case EIO    : ErrorBox("EIO - Input or output"); break;
     case EISDIR : ErrorBox("EISDIR - Is a directory"); break;
     case EMFILE : ErrorBox("EMFILE - Too many open files"); break;
     case EMLINK : ErrorBox("EMLINK - Too many links"); break;
     case ENAMETOOLONG : ErrorBox("ENAMETOOLONG - File name too long"); break;
     case ENFILE : ErrorBox("ENFILE - Too many open files in system"); break;
     case ENODEV : ErrorBox("ENODEV - No such device"); break;
     case ENOENT : ErrorBox("ENOENT - No such file or directory"); break;
     case ENOEXEC: ErrorBox("ENOEXEC - Unable to execute file"); break;
     case ENOSPC : ErrorBox("ENOSPC - No space left on drive"); break;
     case ENOSYS : ErrorBox("ENOSYS - Function not implemented");
     case ENOTDIR: ErrorBox("ENOTDIR - Not a directory"); break;
     case ENOTEMPTY : ErrorBox("ENOTEMPTY - Directory not empty");  break;
     case ENOTTY : ErrorBox("ENOTTY - Inappropriate I/O control operation"); break;
     case ENXIO  : ErrorBox("ENXIO - No such device or address"); break;
     case EPERM  : ErrorBox("EPERM - Operation not permitted"); break;
     case EPIPE  : ErrorBox("EPIPE - Broken pipe"); break;
     case EROFS  : ErrorBox("EROFS - Read-only file system"); break;
     case ESPIPE : ErrorBox("ESPIPE - Invalid seek"); break;
     case ESRCH  : ErrorBox("ESRCH - No such process"); break;
     case EXDEV  : ErrorBox("EXDEV - Improper link"); break;
     //case ENMFILE: ErrorBox("ENMFILE - No more files"); break;

     default : ErrorBox("Undefined error!"); break;
   }
}
