/***************************************************************************
              inifile.cpp  -  Handling of INI-style textfiles
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by MoP
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <fstream>
#include "inifile.h"
#include "fm_str.h"

#define ENCRYPTMARKER '@'

cINIFile::cINIFile(string fname, int openmode/*=0*/) {
  name=fname;
  mode=openmode;
  if (openmode==faRead) {
    ReadLines();
    return;
  }
  if (openmode==faWrite) {
    return;
  }
  error=0;
  encrypt_key=123;
}

cINIFile::cINIFile(char *fname, int openmode/*=0*/) {
  name=fname;
  mode=openmode;
  if (openmode==faRead) {
    ReadLines();
    return;
  }
  if (openmode==faWrite) {
    return;
  }
  error=true;
}

cINIFile::~cINIFile() {

}

void cINIFile::ReadLines() {
  ifstream f(name.c_str());
  if (f.fail()) {
    error=true;
  } else {
    char temp[255];
    string s;
    while(!f.eof()) {
      f.getline(temp,sizeof(temp));
      s=temp;
      if (s[0]!='#' && s[0]!=';') {
        lines+=s;
        touched+=false;
      }
    }
    f.close();
  }
}

void cINIFile::WriteLines() {
  if (mode==faWrite) {
    if (!lines.Size()) return;
    ofstream f(name.c_str());
    if (f.fail()) {
      error=true;
      return;
    }
    for (int i=0; i<lines.Size(); i++) {
      f <<lines[i] <<endl;
    };
    f.close();
  }
}


bool cINIFile::Flush() {
  WriteLines();
  return error;
}

string cINIFile::FindValue(const string& ident) {
  if (lines.Empty()) return "";
  string s;
  for(int i=0; i<lines.Size(); i++) {
    if (lines[i].substr(0,ident.length())==ident && !touched[i]) {
      string temp=lines[i].substr(ident.length()+1,999);
      //StripChar(temp,' ');
      StripChar(temp,'=');
//      lines-=i; //i--;
      touched[i]=true;
      return temp;
    }
  }
  s="";
  return s;
}

int cINIFile::GetValue(const string& ident, int def) {
  string temp=FindValue(ident);
  int i=atoi(temp.c_str());
  if (i!=0) return i; else return def;
}

bool cINIFile::GetValue(const string& ident, bool def) {
  string temp=FindValue(ident);
  temp=Upcase(temp);
  if (temp=="YES" || temp=="TRUE") return true;
  if (temp=="NO" || temp=="FALSE") return false;
  return def;
}

string cINIFile::GetValue(const string& ident, const string& def) {
  string temp=FindValue(ident);

  if(temp[0]==ENCRYPTMARKER) {
    string s="";
    for(int i=1; i<temp.length(); i++) {
      char ch=temp[i];
      ch=ch^encrypt_key;
      s=s+ch;
    }
    temp=s;
  }

  if (temp!="") return temp; else return def;
}

string cINIFile::GetValue(const string& ident, const char* def) {
  string s=def;
  return GetValue(ident,s);
}

void cINIFile::SetValue(const string& ident, int value) {
  lines+=ident+"="+MkStr(value);
}

void cINIFile::SetValue(const string& ident, bool value) {
  lines+=ident+"="+MkStr(value);
}

void cINIFile::SetValue(const string& ident, const string& value,bool writeifempty/*=false*/) {
  if (!writeifempty && value=="") return;
  lines+=ident+"="+value;
}


void cINIFile::SetValueEncrypted(const string& ident, const string& value,bool writeifempty/*=false*/) {
  if (!writeifempty && value=="") return;

  string temp="";
  for(int i=0; i<value.length(); i++) {
    char ch=value[i]; ch=ch^encrypt_key;
    temp=temp+ch;
  }

  lines+=ident+"="+ENCRYPTMARKER+temp;
}


void cINIFile::SetBlank() {
  lines+="";
}

void cINIFile::SetComment(const string& comment/*=""*/) {
  if (comment!="") lines+=("#"+comment); else SetBlank();
}
