/***************************************************************************
                          msocket.cpp  -  description
                             -------------------
    begin                : Thu Aug 17 2000
    copyright            : (C) 2000 by Morten Poulsen
    email                : sc@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "msocket.h"
#include <string.h>
#include <unistd.h>

//******** BASICS
#ifdef _SOCKET_SUPPORT_

sockBase::sockBase(int _port, sockErrorFunction efunc=NULL) {
  port=_port;
  errorfunc=efunc;
}

void sockBase::SetErrorFunc(sockErrorFunction _efunc) {
  errorfunc=_efunc;
}

void sockBase::sockError(char *errorstring, int errorcode) {
  if (errorfunc) errorfunc(errorstring, errorcode);
}


//******** CLIENT

sockClient::sockClient(char* _host, int _port, sockErrorFunction efunc)
  : sockBase(_port, efunc) {

  host=_host;
  if ((server_host_name=gethostbyname(host))==0) {
    sockError("Error resolving hostname.",0);
    return;
  }
  bzero(&pin, sizeof(pin));
  pin.sin_family=AF_INET;
  pin.sin_addr.s_addr=htonl(INADDR_ANY);
  pin.sin_addr.s_addr=((struct in_addr*)(server_host_name->h_addr))->s_addr;
  pin.sin_port=htons(port);
}

char *sockClient::GetResponseFromServer(char *command) {
  if ((socket_descriptor=socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    sockError("Error opening socket.");
    return NULL;
  }
  if (connect(socket_descriptor, (const sockaddr*)&pin, sizeof(pin)) == -1) {
    sockError("Error connecting to socket.");
    return NULL;
  }
  if (send(socket_descriptor, command, strlen(command), 0) == -1) {
    sockError("Error sending.");
    return NULL;
  }
  if (recv(socket_descriptor, buf, _MAXBLOCKSIZE, 0) == -1) {
    sockError("Error receiving response.");
    return NULL;
  }
  close(socket_descriptor);
  return buf;
}


//******** SERVER

sockServer::sockServer(sockServerWorkFunc _workfunc, int _port, sockErrorFunction efunc)
  : sockBase(_port, efunc) {
  workfunc=_workfunc;
  unsigned int address_size;
  int i, len;
  char buf[_MAXBLOCKSIZE];
  temp_buf=new char[_MAXBLOCKSIZE];
  if ((socket_descriptor=socket(AF_INET, SOCK_STREAM, 0))==-1) {
    sockError("Error opening socket.");
    return;
  }

  bzero(&sin, sizeof(sin));
  sin.sin_family=AF_INET;
  sin.sin_addr.s_addr=INADDR_ANY;
  sin.sin_port=htons(port);

  if (bind(socket_descriptor, (struct sockaddr*)&sin, sizeof(sin))==-1) {
    sockError("Error binding socket.");
    return;
  }

  if (listen(socket_descriptor, 20)==-1) {
    sockError("Error listening to socket.");
    return;
  }

  while (47) {
    temp_socket_descriptor=accept(socket_descriptor, (struct sockaddr*)&pin, &address_size);
    if (temp_socket_descriptor==-1) {
      sockError("Error accepting connection.");
      return;
    }

    if (recv(temp_socket_descriptor, buf, _MAXBLOCKSIZE, 0)==-1) {
      sockError("Error recieving request.");
      return;
    }
    exitnow=false;
    workfunc(buf,temp_buf, _MAXBLOCKSIZE,exitnow);
    if (exitnow) return;

    //This means only text can be transferred, right?
    len=strlen(temp_buf);

    if (send(temp_socket_descriptor, temp_buf, len, 0)==-1) {
      sockError("Error sending respons.");
      return;
    }

    close(temp_socket_descriptor);
  }  
}

sockServer::~sockServer() {
  delete temp_buf;
}


#endif //_SOCKET_SUPPORT_
