#include "fm_debug.h"
#include "fm_conf.h"
#include "fm_vfs.h"
#include "sc_ftp.h"
#include "fm_entry.h"
#include "fm_window.h"
#include "ftplib.h"
#include "fm_textfile.h"
#include "sc_file.h"
#include "target.h"

#ifdef _FTP_POSSIBLE_

bool ConnectToSite(const string& site, netbuf **ftpbuf) {
  for (int i=0; i<Configuration.ftpsites.size(); i++) {
    if (Configuration.ftpsites[i].ftpsite==site) {
      cFtpSite site=Configuration.ftpsites[i];

      WriteTopMessage("Connecting to site...");
      if (FtpConnect(site.ftpsite.c_str(),ftpbuf)==0) return false;

      WriteTopMessage("Logging in...");
      if (FtpLogin(site.username.c_str(), site.password.c_str(), *ftpbuf)==0) {
        return false;
      }

      if (site.initialdir!="" && site.initialdir!="/") {
        WriteTopMessage("Changing to initial dir...");
        if (FtpChdir(site.initialdir.c_str(), *ftpbuf) ==0) {
          return false;
        }
      }
      
      return true;
    }
  }
  return false;
}



class cFileOnFtp : public cFile {
  public:
    cFileOnFtp(const sParsedDir& spd) : cFile(spd.completename) {


    }
    ~cFileOnFtp() {
      FtpQuit(MyFtp);
      //CleanUpFtp();
    }

    bool Open(tOpenMode om) {
      //parse the bastard
      sParsedDir spd=PleaseParseDir(filename);

       //Try to connect
      if (!ConnectToSite(spd.fullname,&MyFtp)) return false;

      int mode;
      if (om==omRead  || om==omGZRead)  mode=FTPLIB_FILE_READ;
      if (om==omWrite || om==omGZWrite) mode=FTPLIB_FILE_WRITE;

      string ftpname="./"+spd.arc_fullname;

      if (FtpAccess(ftpname.c_str(), mode,
                                     FTPLIB_BINARY,
                                     MyFtp, &MyData)==0) {
        return false;
      }

      return true;
    }
    int Read(void *buf,int size) {
      return FtpRead(buf,size,MyData);
    }
    int Write(void *buf,int size) {
      return FtpWrite(buf,size,MyData);
    }
    void Close() {
      FtpClose(MyData);
    }
    bool Delete() {
    }
    int Size() {
    }
    int Pos() {
    }
    int GetMode() {
    }
    bool SetMode(int) {
      return false;
    }

  private:
    netbuf *MyData;
    netbuf *MyFtp;
};



cFile* GetFtpFilePointer(const sParsedDir& spd) {
  return new cFileOnFtp(spd);
}




bool _EditFtpSite(cFtpSite& editsite) {
  cFtpSite ftpsite=editsite;
  tKey k;
  cWindow w(ScreenW()-10,14,"Enter new ftp-site");
  do {
    w.RedrawBox();
    w.Write(1,1,"~1 Ftp-address:",cYELLOW);
    w.Write(3,2,ftpsite.ftpsite);
    w.Write(1,3,"~2 Initial directory:",cYELLOW);
    w.Write(3,4,ftpsite.initialdir);
    w.Write(1,5,"~3 Username:",cYELLOW);
    w.Write(3,6,ftpsite.username);
    w.Write(1,7,"~4 Password:",cYELLOW);
    w.Write(3,8,ftpsite.password);
    w.Write(1,9,"~5 Use passive mode:",cYELLOW);
    w.Write(3,10,((ftpsite.passivemode) ? "passive" : "not passive"));

    w.Write(1,12,"~1-~5 change value");
    w.Write(1,13,"~E~n~t~e~r confirms  ~E~S~C Cancel",cYELLOW);
    k=GetSomeKey();
    switch (k) {
      case k1 : {
        string s=ftpsite.ftpsite; if (InputWindow("Ftp-address (w/o dirs)",s,40)) ftpsite.ftpsite=s;
        break;
      }
      case k2 : {
        string s=ftpsite.initialdir; if (InputWindow("Initial directory",s,40)) ftpsite.initialdir=s;
        break;
      }
      case k3 : {
        string s=ftpsite.username; if (InputWindow("Username",s,40)) ftpsite.username=s;
        break;
      }
      case k4 : {
        string s=ftpsite.password; if (InputWindow("Password",s,40)) ftpsite.password=s;
        break;
      }
      case k5 : {
        ftpsite.passivemode=(!ftpsite.passivemode);
        break;
      }
    }
  } while (!KeyIn(k,kESC,kEnter));
  if (k==kEnter) {
    editsite=ftpsite;
    return true;
  }
  return false;
}

bool EnterNewFtpSite() {
  tKey k;
  cFtpSite site;
  site.initialdir="/",
  site.username="anonymous",
  site.password=Configuration.anon_password;
  site.passivemode=false;
  if (!InputWindow("Ftp-address (w/o dirs)",site.ftpsite,ScreenW()-12)) return false;
  if (_EditFtpSite(site)) {
    Configuration.ftpsites.push_back(site);
    return true;
  }
  return false;
}


bool EditFtpSite(int sitenumber) {
  if (sitenumber<0 || sitenumber>=Configuration.ftpsites.size()) return false;
  return _EditFtpSite(Configuration.ftpsites[sitenumber]);
}



bool DeleteFtpSite(int sitenumber) {
  if (sitenumber<0 || sitenumber>=Configuration.ftpsites.size()) return false;
  switch (PromptBox("Confirmation","Are you sure? (y/~n)",kY,kN,kEnter,kESC)) {
    case kY : break;
    default : return false;
  }
  Configuration.ftpsites-=sitenumber;
  return true;
  /*for (int i=0; i<Configuration.ftpsites.size(); i++) {
    if (Configuration.ftpsites[i].ftpsite==site) {
      Configuration.ftpsites-=i;
      return true;
    }
  }*/
  //return false; ///hmmm, something is wrong!
}

void FtpIdleAction() {
  ShowRotator();
};


string GetNextWord(string& s) {
  string temp="";
  int i=0;
  while (s[i]==' ' && i<s.length()) i++;
  if (i==s.length()) return "";

  while (s[i]!=' ' && i<s.length()) {
    temp+=s[i];
    i++;
  }
  if (i==s.length()) {
    s="";
    return temp;
  } else {
    s=s.substr(i,9999);
    return temp;
  }
}

bool ParseFtpEntry(const string& entry, const string& parentdir, cDirEntry& de, bool onlydirs=false) {
  string s=entry;
  if (s[0]!='d' && onlydirs) return false; //unwanted files excluded
  string temp;

  int modeattr=0;

  //modedescription
  if ((temp=GetNextWord(s))=="") return false;
  if (temp=="total" || temp.length()<10) return false;
//  #if
  if (temp[0]=='d') modeattr=modeattr | S_IFDIR;

//  if (temp[0]=='l') modeattr=modeattr | __S_IFDIR;
  //Not entirely correct, but mostly

  if (temp[1]!='-') modeattr=modeattr | S_IRUSR;
  if (temp[2]!='-') modeattr=modeattr | S_IWUSR;
  if (temp[3]!='-') modeattr=modeattr | S_IXUSR;

  if (temp[4]!='-') modeattr=modeattr | S_IRGRP;
  if (temp[5]!='-') modeattr=modeattr | S_IWGRP;
  if (temp[6]!='-') modeattr=modeattr | S_IXGRP;

  if (temp[7]!='-') modeattr=modeattr | S_IROTH;
  if (temp[8]!='-') modeattr=modeattr | S_IWOTH;
  if (temp[9]!='-') modeattr=modeattr | S_IXOTH;


  
  //some number
  if ((temp=GetNextWord(s))=="") return false;

  //user ID
  if ((temp=GetNextWord(s))=="") return false;

  //group ID
  if ((temp=GetNextWord(s))=="") return false;

  //Size
  if ((temp=GetNextWord(s))=="") return false;
//  int size=777;
  int size=atoi(temp.c_str());


  //Month
  if ((temp=GetNextWord(s))=="") return false;
  int time=0;


  //Date
  if ((temp=GetNextWord(s))=="") return false;

  //Year/time
  if ((temp=GetNextWord(s))=="") return false;

  //Name
  if ((temp=GetNextWord(s))=="") return false;
  string name=temp;

  de.MakeNormal(name, parentdir+name, useMode, modeattr, time,size);

  //Symlink
  /*if ((temp=GetNextWord(s))=="") {
    //Symlink target
    if ((temp=GetNextWord(s))=="");
  }*/



   return true;
}

bool FtpScan(tDirContainer& tdc, const string& dir, bool onlydirs=false) {
  //Handle the root of all ftp-sites
  if (StripSlash(dir)==_FTPTOKEN) {
    tdc.Clear();
    cDirEntry tde;
    if (!onlydirs) { //Skip .. entry when calling from tree
      tde.MakeSpecial("..","..",true);
      tdc+=tde;
    }
    for (int i=0; i<Configuration.ftpsites.size(); i++) {
      cFtpSite fs=Configuration.ftpsites[i];
      tde.MakeFtpSite(fs.ftpsite,"["+fs.ftpsite+"]");
      tde.isftpsite=true;
      tdc+=tde;
    }
    return true;
  }

  //parse the bastard
  sParsedDir spd=PleaseParseDir(AddSlash(dir));


  netbuf *MyFtp;
  //Try to connect
  if (!ConnectToSite(spd.fullname,&MyFtp)) return false;

  //Get a dirlisting to a temp file
  string tempname;
/*  if (IsDebuggingEnabled()) {
    tempname="/root/temp.lst"; //Or for denugging purposes a known file
  } else {
    tempname=GetTempFileName();
  }*/
  tempname=GetTempFileName();

  string dirname=StripSlash(spd.arc_dirname)+"/.";

  WriteTopMessage("Getting filelist...");
  if (FtpDir(tempname.c_str(), dirname.c_str(), MyFtp)==0) {
    FtpQuit(MyFtp);
    return false;
  }

  WriteTopMessage("Parsing filelist...");
  cTextFile tf;
  if (!tf.ReadWholeFile(tempname)) {
    DeleteFile(tempname);
    FtpQuit(MyFtp);
    return false;
  }
  DeleteFile(tempname);
  cDirEntry tde;
  tdc.Clear();

  //Skip .. entry when calling from tree
  if (!onlydirs) {
    tde.MakeSpecial("..","..",true);
    tdc+=tde;
  }


  //Parse the temp file
  string parentdir=AddSlash(dir);
  parentdir=StrReplace(parentdir,"]/","]");// ouch, ugly hack!
  for (int i=0; i<tf.Size(); i++) {

    if (ParseFtpEntry(tf[i], parentdir, tde, onlydirs)) {
      if (tde.name=="..") continue; //I handle this myself!
      if (tde.name==".") continue;  //Phew, get away!
      tdc+=tde;
    }
  }
  FtpQuit(MyFtp);
  return true;
}


bool FtpDoesDirExist(const string& s) {
  if (s==_FTPTOKEN) return true;
  return true; //hhhmm, a check would be really slow...
}

bool FtpDoesFileExist(const string&) {
  return false;
}


bool FtpRename(const string& n1, const string& n2) {
  //parse the bastard
  sParsedDir spd1=PleaseParseDir(AddSlash(n1));
  sParsedDir spd2=PleaseParseDir(AddSlash(n2));

  netbuf *MyFtp;
  //Try to connect
  if (!ConnectToSite(spd1.fullname,&MyFtp)) return false;

  bool result=FtpRename(spd1.arc_fullname.c_str(), spd2.arc_fullname.c_str(), MyFtp);
  FtpQuit(MyFtp);
  //CleanUpFtp();
  return result;
}

bool FtpSetFileMode(const string&name, const sModeChanges& md) {
  return false;
}

bool FtpDeleteFile(const string& name) {
  //parse the bastard
  sParsedDir spd=PleaseParseDir(name);

  netbuf *MyFtp;

  //Try to connect
  if (!ConnectToSite(spd.fullname,&MyFtp)) return false;

  bool result=FtpDelete(spd.arc_fullname.c_str(), MyFtp);
  FtpQuit(MyFtp);
  return result;
}

bool FtpMkDir(const string& dir) {
  //parse the bastard
  sParsedDir spd=PleaseParseDir(AddSlash(dir));

  netbuf *MyFtp;
  //Try to connect
  if (!ConnectToSite(spd.fullname,&MyFtp)) return false;

  bool result=FtpMkdir(spd.arc_fullname.c_str(), MyFtp);
  FtpQuit(MyFtp);
  return result;
}

bool FtpRmDir(const string& dir) {
  //parse the bastard
  sParsedDir spd=PleaseParseDir(AddSlash(dir));

  netbuf *MyFtp;
  //Try to connect
  if (!ConnectToSite(spd.fullname,&MyFtp)) return false;

  bool result=FtpRmdir(spd.arc_fullname.c_str(), MyFtp);
  FtpQuit(MyFtp);
  return result;
}


#endif // _FTP_POSSIBLE_
