#include <string>
#include "fm_menu.h"
#include "fm_conf.h"
#include "fm_editcon.h"
#include "fm_viewcon.h"
#include "sc_usermenu.h"
#include "fm_vfs.h"
#include "sc_shell.h"
#include "target.h"
#include "fm_entry.h"

void ShowUserMenu(const tDirContainer files, const string& currentdir, const string& currentfile) {
  cMenu m("UserMenu");
  if (Configuration.menuopts.Size()==0) {
    MessageBox("Sorry, no menuoptions defined!");
    return;
  }
  for(int i=0; i<Configuration.menuopts.Size(); i++) {
    m+=Configuration.menuopts[i].showline;
  }
  int i=m.Run();   //Show the menu
  if (!i) return;  //If menu were aborted (=0) then exit
  i--;             //decrease i for array-indexing
  string comline=Configuration.menuopts[i].commandline;
  #ifdef _TARGET_LINUX_
    string homedir=getenv("HOME");
    comline=StrReplace(comline,"~",homedir);
  #endif
  comline=StrReplace(comline,"%f",currentfile);
  comline=StrReplace(comline,"%d",currentdir);
  if (comline.find("%p")!=-1) {
    string s="";
    if (InputWindow("Prompt:",s,ScreenW()-10)) {
      comline=StrReplace(comline,"%p",s);
    }
  }

  string listname="";
  if (comline.find("%l")!=-1) {
    cTextFile c;
    #ifdef _TARGET_UNIX_
      c.SetFlavor(Unix);
    #else
      c.SetFlavor(DOS);
    #endif
    for (int i=0; i<files.size(); i++) {
      if (files.Read(i).tagged) {
        if (files.Read(i).isdir) {
          c+=files.Read(i).name+"/";
        } else {
          c+=files.Read(i).name;
        }
      }
    }
    listname=GetTempFileName();
    c.WriteToFile(listname);
    comline=StrReplace(comline,"%l",listname);
  }

  switch (Configuration.menuopts[i].type) {
    case maEdit : {
      DedicatedEdit(comline);
      break;
    }
    case maView : {
      DedicatedView(comline);
      break;
    }
    case maMulti : {
      for (int i=0; i<files.size(); i++) {
        if (files.Read(i).tagged) {
          comline=StrReplace(Configuration.menuopts[i].commandline,"%f",files.Read(i).fullname);
          comline=StrReplace(comline,"%d",currentdir);
          ExecuteCommand(comline);
        }
      }
      break;
    }
    case maCapture : {
      ShowMessage("Executing: "+comline);
      string outname=GetTempFileName();
      ExecuteCommand(comline+" > "+outname,true);
      DedicatedView(outname);
      DeleteFile(outname);
      break;
    }
    default : ExecuteCommand(comline);
  }
  DeleteFile(listname);
}
