#include "syntax_defs.h"

cParser::cParser() {
  cline=0;
  cchar=0;
  casesensitive=true;
}

char cParser::GetNextch(cTextFile& lines) {
  char c;
  if (cline==lines.Size()) { //Past last line?
    c=_eof;
  } else {
    if (cchar==lines[cline].length()) { //past last char?
      c=_eol;
      cchar=0;
      cline++;
    } else {
      c=lines[cline][cchar];
      cchar++;
    }
  }
  return c;
}

char cParser::PeekNextch(cTextFile& lines) {
  char c;
  if (cline==lines.Size()) { //Past last line?
    c=_eof;
  } else {
    if (cchar==lines[cline].length()) { //past last char?
      c=_eol;
    } else {
      c=lines[cline][cchar];
    }
  }
  return c;
}

void cParser::Add(char ch, int col=-1) {
  if (col!=-1) {
    AddCol(col);
  }
  temp=(temp+ch);
}

void cParser::AddCol(int col) {
  if (col==currentcolor && temp!="") return; //Should optimize things a bit
  Add(_col);
  if (col==0) Add(0x00); else Add(col);
  currentcolor=col;
}

void cParser::AddBuf() {
  if (IsReserved(buf)) {
    AddCol(COL_RESERVED);
  } else {
    AddCol(COL_IDENT);
  }
  temp+=buf;
  //AddCol(COL_DEFAULT);
  buf="";
}

bool cParser::IsReserved(const string& s) {
  int i=0;
  do {
    if (casesensitive) {
      if (strcmp(s.c_str(),reserved_words[i].c_str())==0) return true;
    } else {
      if (strcasecmp(s.c_str(),reserved_words[i].c_str())==0) return true;
    }
    i++;
  } while (reserved_words[i]!="");
  return false;
}


int cParser::Classify(char c) {
  if (c==' ') return __SPACE;
  if (c==_eol) return __EOL;
  if (c=='_') return __CHAR; //Usually allowed in identifiers
  if (c>='0' && c<='9') return __NUMBER;
  if (c>='[' && c<='`') return __SYMBOL;
  if (c>='!' && c<='/') return __SYMBOL;
  if (c>='{' && c<='~') return __SYMBOL;
  if (c>=':' && c<='@') return __SYMBOL;
  if (c>'~') return __EXTCHAR;
  if (c==_eof) return __EOF;
  return __CHAR;
}
