/*f10.dk
brugernavn: cola
password: cola*/

#include "syntax_pascal.h"
#include "syntax_defs.h"

const int __default = 0;
const int __comment  = 2; //Comment spanning several lines
const int __string   = 4; //String spanning several lines
//notused
//const int __stringl  = 5; //string ending with a EOL
const int __ident   = 6;
const int __symbols = 7;
const int __literal = 8;
const int __preproc = 9;

string pascal_reserved_words[] = {
  "procedure", "function", "begin", "end", "integer",
  "unit","interface","implementation","string","for",
  ""
};


class cParserPascal : cParser {
  public:
    cParserPascal() {
      temp=""; buf="";
      int i=0;
      while (pascal_reserved_words[i]!="") {
        reserved_words+=pascal_reserved_words[i];
        i++;
      }
      reserved_words+="";
      casesensitive=false;
      
    };
    void SyntaxParse(cTextFile& lines);
};


void cParserPascal::SyntaxParse(cTextFile& lines) {
  string ts="";
  int state=__default;
  char cc;
  do {
    do {cc=GetNextch(lines);} while (!cc); //disallow 0x00
    switch (Classify(cc)) {
      case __CHAR : {
        switch (state) {
          case __symbols :
          case __default : {
            buf=cc;
            state=__ident;
            break;
          }
          case __ident : {
            buf=(buf+cc);
            break;
          }

          default : {
            Add(cc);
            break;
          }
        }
        break;
      };
    
      case __NUMBER : {
        switch (state) {
          case __default :
          case __symbols : {
            Add(cc,COL_LITERAL);
            state=__literal;
            break;
          }
          case __ident : {
            AddBuf(); //numbers can follow chars in idents
            break;
          }

          default : {
            Add(cc);
          }
        }
        break;
      }

      case __SYMBOL : {
        switch (state) {
          case __default : {
            switch (cc) {
              case '#' : {
                Add(cc,COL_PREPROC);
                state=__preproc;
                break;
              }
              case '(' : {
                if (PeekNextch(lines)=='*') {
                  Add(cc,COL_COMMENT);
                  state=__comment;
                  break;                  
                }
                Add(cc,COL_SYMBOL);
                break;
              }
              
              case '{' : {
                Add(cc,COL_COMMENT);
                state=__comment;
                break;
              }
              case '\"' : //Not used in Pascal, but just for all sakes
              case '\'' : { //String beginning
                Add(cc,COL_STRING);
                state=__string;
                break;
              }
              default : {
                state=__symbols;
                Add(cc,COL_SYMBOL);
              }
            }
            break;
          }
          case __string : {
            switch (cc) {
              case '\"' : //String ending
              case '\'' : {
                Add(cc); AddCol(COL_DEFAULT);
                state=__default;
                break;
              }

              default : {
                Add(cc);
              }
            }
            break;
          }
          case __comment : {
            if (cc=='}' || (cc=='*' && PeekNextch(lines)==')')) { //end of comment?
              Add(cc); Add(GetNextch(lines));
              AddCol(COL_DEFAULT);
              state=__default;
              break;
            }

            Add(cc);
            break;
          }
          
          case __ident : {
            AddBuf(); //fallthrough intensional
            
          }
          case __literal : {
            state=__symbols;
            Add(cc,COL_SYMBOL);
            break;
          }

          default : {
            Add(cc,COL_SYMBOL);
          }
        }
        break;
      }

      case __SPACE : {
        switch (state) {
          case __ident : {
            state=__default;
            AddBuf(); //fallthrough intensional
            AddCol(COL_DEFAULT);
          }
          default : {
            Add(cc);
          }
        }
        
        break;
      }

      case __EOL : {
        switch (state) {
          case __comment :
          case __string : {
            //Do not reset to default at end of line
            break;
          }
          case __ident : {
            AddBuf(); //fallthrough intensional
          }

          default : {
            state=__default;
            currentcolor=COL_DEFAULT;
          }
        }
        lines[cline-1]=temp; //Replace with colorcoded line
        temp="";
        AddCol(currentcolor);
        break;
      }
      default : ; //at EOF do nothing particular
    }
  } while (cc!=_eof);
  if (temp.length()>2) {
    lines[cline-1]=temp;
  }
}


bool DoPascalSyntax(cTextFile& lines) {
  cParserPascal parser;
  parser.SyntaxParse(lines);
  return true;
}


