/***************************************************************************
                   unicon.h  -  crossplatform console I/O
                             -------------------
    begin                : Mon Jan 3 2000
    copyright            : (C) 2000 by Vladi B-S (minor modifications by MoP)
    email                : fmpro@poulsen.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
 *
 * (c) Vladi Belperchinov-Shabanski "Cade" <cade@biscom.net> 1998-1999
 *
 * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
 *
 */

#ifndef _UNICON_H_
#define _UNICON_H_

#include "target.h"

#ifdef _TARGET_UNKNOWN_
  #error "I don't know what is the target platform"
#endif

#ifdef _TARGET_TEXTIO_CURSES_
  #if defined(_TARGET_LINUX_)
    #include <curses.h>
    #undef erase()
    #include <pty.h>
  #elif defined(_TARGET_NETBSD_)
    #include <ncurses.h>
  #elif defined(_TARGET_CYGWIN_)
    #include <curses.h>
    #include <termios.h>
    #include <sys/ioctl.h>
    #include <ctype.h>
  #else
    #include <curses.h>
  #endif
#endif


/*#ifdef _TARGET_TEXTIO_CURSES_
  #if defined(_TARGET_LINUX_)
    #include <curses.h>
  #elif defined(_TARGET_NETBSD_)
    #include <ncurses.h>
  #else
    #include <curses.h>
  #endif
#endif*/

#ifdef _TARGET_TEXTIO_CONIO_
    #include <conio.h>
#endif

///////////////////////////////////////////////////////////////////////////
//
// COLOR defines
//

  #define CONCOLOR(f,b) (b*16+f)
  #define COLORFG(t)    (t % 16)
  #define COLORBG(t)    (t / 16)

  #define cNORMAL    7
  #define cBOLD      8

  #define cBLACK     0
  #define cBLUE      1
  #define cGREEN     2
  #define cCYAN      3
  #define cRED       4
  #define cMAGENTA   5
  #define cBROWN     6
  #define cYELLOW    6
  #define cLGRAY     7
  #define chBLACK    7
  #define cWHITE     7
  #define cDGRAY     8
  #define chBLUE     9
  #define chGREEN   10
  #define chCYAN    11
  #define chRED     12
  #define chMAGENTA 13
  #define chYELLOW  14
  #define chWHITE   15

///////////////////////////////////////////////////////////////////////////
//
// UCKEY defines
//

///// common /////

  #define UCKEY_CTRL_A    1
  #define UCKEY_CTRL_B    2
  #define UCKEY_CTRL_C    3
  #define UCKEY_CTRL_D    4
  #define UCKEY_CTRL_E    5
  #define UCKEY_CTRL_F    6
  #define UCKEY_CTRL_G    7
  #define UCKEY_CTRL_H    8
  #define UCKEY_CTRL_I    9
  #define UCKEY_CTRL_J   10
  #define UCKEY_CTRL_K   11
  #define UCKEY_CTRL_L   12
  #define UCKEY_CTRL_M   13
  #define UCKEY_CTRL_N   14
  #define UCKEY_CTRL_O   15
  #define UCKEY_CTRL_P   16
  #define UCKEY_CTRL_Q   17
  #define UCKEY_CTRL_R   18
  #define UCKEY_CTRL_S   19
  #define UCKEY_CTRL_T   20
  #define UCKEY_CTRL_U   21
  #define UCKEY_CTRL_V   22
  #define UCKEY_CTRL_W   23
  #define UCKEY_CTRL_X   24
  #define UCKEY_CTRL_Y   25
  #define UCKEY_CTRL_Z   26


// I don't know if this is just linux problem...
#ifdef _TARGETFLAVOR_UNIX_
  #undef UCKEY_CTRL_Z
  #define UCKEY_CTRL_Z    407  // a hack ?! why ^Z is not plain 26?!
#endif

///// DJGPP /////
#ifdef _TARGET_TEXTIO_CONIO_
  #define UCKEY_PREFIX      1024
  #define UCKEY_BACKSPACE   8
  #define UCKEY_LEFT      (UCKEY_PREFIX + 75)
  #define UCKEY_RIGHT     (UCKEY_PREFIX + 77)
  #define UCKEY_UP        (UCKEY_PREFIX + 72)
  #define UCKEY_DOWN      (UCKEY_PREFIX + 80)
  #define UCKEY_HOME      (UCKEY_PREFIX + 71)
  #define UCKEY_END       (UCKEY_PREFIX + 79)
  #define UCKEY_PPAGE     (UCKEY_PREFIX + 73)
  #define UCKEY_NPAGE     (UCKEY_PREFIX + 81)
  #define KEY_IC        (UCKEY_PREFIX + 82)
  #define KEY_DC        (UCKEY_PREFIX + 83)

  #define UCKEY_F0        (UCKEY_PREFIX + 58)
  #define UCKEY_F(n)      (UCKEY_PREFIX + (58+(n)))

  #define UCKEY_F1        (UCKEY_PREFIX + 59)
  #define UCKEY_F2        (UCKEY_PREFIX + 60)
  #define UCKEY_F3        (UCKEY_PREFIX + 61)
  #define UCKEY_F4        (UCKEY_PREFIX + 62)
  #define UCKEY_F5        (UCKEY_PREFIX + 63)
  #define UCKEY_F6        (UCKEY_PREFIX + 64)
  #define UCKEY_F7        (UCKEY_PREFIX + 65)
  #define UCKEY_F8        (UCKEY_PREFIX + 66)
  #define UCKEY_F9        (UCKEY_PREFIX + 67)
  #define UCKEY_F10       (UCKEY_PREFIX + 68)

  #define UCKEY_SH_F1     (UCKEY_PREFIX +  84)
  #define UCKEY_SH_F2     (UCKEY_PREFIX +  85)
  #define UCKEY_SH_F3     (UCKEY_PREFIX +  86)
  #define UCKEY_SH_F4     (UCKEY_PREFIX +  87)
  #define UCKEY_SH_F5     (UCKEY_PREFIX +  88)
  #define UCKEY_SH_F6     (UCKEY_PREFIX +  89)
  #define UCKEY_SH_F7     (UCKEY_PREFIX +  90)
  #define UCKEY_SH_F8     (UCKEY_PREFIX +  91)
  #define UCKEY_SH_F9     (UCKEY_PREFIX +  92)
  #define UCKEY_SH_F10    (UCKEY_PREFIX +  93)

  #define UCKEY_CTRL_F1   (UCKEY_PREFIX +  94)
  #define UCKEY_CTRL_F2   (UCKEY_PREFIX +  95)
  #define UCKEY_CTRL_F3   (UCKEY_PREFIX +  96)
  #define UCKEY_CTRL_F4   (UCKEY_PREFIX +  97)
  #define UCKEY_CTRL_F5   (UCKEY_PREFIX +  98)
  #define UCKEY_CTRL_F6   (UCKEY_PREFIX +  99)
  #define UCKEY_CTRL_F7   (UCKEY_PREFIX + 100)
  #define UCKEY_CTRL_F8   (UCKEY_PREFIX + 101)
  #define UCKEY_CTRL_F9   (UCKEY_PREFIX + 102)
  #define UCKEY_CTRL_F10  (UCKEY_PREFIX + 103)

  #define UCKEY_ALT_F1    (UCKEY_PREFIX + 104)
  #define UCKEY_ALT_F2    (UCKEY_PREFIX + 105)
  #define UCKEY_ALT_F3    (UCKEY_PREFIX + 106)
  #define UCKEY_ALT_F4    (UCKEY_PREFIX + 107)
  #define UCKEY_ALT_F5    (UCKEY_PREFIX + 108)
  #define UCKEY_ALT_F6    (UCKEY_PREFIX + 109)
  #define UCKEY_ALT_F7    (UCKEY_PREFIX + 110)
  #define UCKEY_ALT_F8    (UCKEY_PREFIX + 111)
  #define UCKEY_ALT_F9    (UCKEY_PREFIX + 112)
  #define UCKEY_ALT_F10   (UCKEY_PREFIX + 113)

  #define UCKEY_ALT_1     (UCKEY_PREFIX + 120)
  #define UCKEY_ALT_2     (UCKEY_PREFIX + 121)
  #define UCKEY_ALT_3     (UCKEY_PREFIX + 122)
  #define UCKEY_ALT_4     (UCKEY_PREFIX + 123)
  #define UCKEY_ALT_5     (UCKEY_PREFIX + 124)
  #define UCKEY_ALT_6     (UCKEY_PREFIX + 125)
  #define UCKEY_ALT_7     (UCKEY_PREFIX + 126)
  #define UCKEY_ALT_8     (UCKEY_PREFIX + 127)
  #define UCKEY_ALT_9     (UCKEY_PREFIX + 128)
  #define UCKEY_ALT_0     (UCKEY_PREFIX + 129)
  #define UCKEY_ALT_MINUS (UCKEY_PREFIX + 130)
  #define UCKEY_ALT_EQ    (UCKEY_PREFIX + 131)

  #define UCKEY_ALT_Q     (UCKEY_PREFIX + 16)
  #define UCKEY_ALT_W     (UCKEY_PREFIX + 17)
  #define UCKEY_ALT_E     (UCKEY_PREFIX + 18)
  #define UCKEY_ALT_R     (UCKEY_PREFIX + 19)
  #define UCKEY_ALT_T     (UCKEY_PREFIX + 20)
  #define UCKEY_ALT_Y     (UCKEY_PREFIX + 21)
  #define UCKEY_ALT_U     (UCKEY_PREFIX + 22)
  #define UCKEY_ALT_I     (UCKEY_PREFIX + 23)
  #define UCKEY_ALT_O     (UCKEY_PREFIX + 24)
  #define UCKEY_ALT_P     (UCKEY_PREFIX + 25)

  #define UCKEY_ALT_A     (UCKEY_PREFIX + 30)
  #define UCKEY_ALT_S     (UCKEY_PREFIX + 31)
  #define UCKEY_ALT_D     (UCKEY_PREFIX + 32)
  #define UCKEY_ALT_F     (UCKEY_PREFIX + 33)
  #define UCKEY_ALT_G     (UCKEY_PREFIX + 34)
  #define UCKEY_ALT_H     (UCKEY_PREFIX + 35)
  #define UCKEY_ALT_J     (UCKEY_PREFIX + 36)
  #define UCKEY_ALT_K     (UCKEY_PREFIX + 37)
  #define UCKEY_ALT_L     (UCKEY_PREFIX + 38)

  #define UCKEY_ALT_Z     (UCKEY_PREFIX + 44)
  #define UCKEY_ALT_X     (UCKEY_PREFIX + 45)
  #define UCKEY_ALT_C     (UCKEY_PREFIX + 46)
  #define UCKEY_ALT_V     (UCKEY_PREFIX + 47)
  #define UCKEY_ALT_B     (UCKEY_PREFIX + 48)
  #define UCKEY_ALT_N     (UCKEY_PREFIX + 49)
  #define UCKEY_ALT_M     (UCKEY_PREFIX + 50)

#endif

///// UNIX /////

#ifdef _TARGET_TEXTIO_CURSES_
  #define UCKEY_unicon_F0        KEY_F(0)
  #define UCKEY_F1        (KEY_F(0) + 1)
  #define UCKEY_F2        (KEY_F(0) + 2)
  #define UCKEY_F3        (KEY_F(0) + 3)
  #define UCKEY_F4        (KEY_F(0) + 4)
  #define UCKEY_F5        (KEY_F(0) + 5)
  #define UCKEY_F6        (KEY_F(0) + 6)
  #define UCKEY_F7        (KEY_F(0) + 7)
  #define UCKEY_F8        (KEY_F(0) + 8)
  #define UCKEY_F9        (KEY_F(0) + 9)
  #define UCKEY_F10       (KEY_F(0) + 10)

  #define UCKEY_SH_F1     (KEY_F(0) + 11)
  #define UCKEY_SH_F2     (KEY_F(0) + 12)
  #define UCKEY_SH_F3     (KEY_F(0) + 13)
  #define UCKEY_SH_F4     (KEY_F(0) + 14)
  #define UCKEY_SH_F5     (KEY_F(0) + 15)
  #define UCKEY_SH_F6     (KEY_F(0) + 16)
  #define UCKEY_SH_F7     (KEY_F(0) + 17)
  #define UCKEY_SH_F8     (KEY_F(0) + 18)
  #define UCKEY_SH_F9     (KEY_F(0) + 19)
  #define UCKEY_SH_F10    (KEY_F(0) + 20)

  #define UCKEY_CTRL_F1   (-1)
  #define UCKEY_CTRL_F2   (-1)
  #define UCKEY_CTRL_F3   (-1)
  #define UCKEY_CTRL_F4   (-1)
  #define UCKEY_CTRL_F5   (-1)
  #define UCKEY_CTRL_F6   (-1)
  #define UCKEY_CTRL_F7   (-1)
  #define UCKEY_CTRL_F8   (-1)
  #define UCKEY_CTRL_F9   (-1)
  #define UCKEY_CTRL_F10  (-1)

  #define UCKEY_ALT_F1    (-1)
  #define UCKEY_ALT_F2    (-1)
  #define UCKEY_ALT_F3    (-1)
  #define UCKEY_ALT_F4    (-1)
  #define UCKEY_ALT_F5    (-1)
  #define UCKEY_ALT_F6    (-1)
  #define UCKEY_ALT_F7    (-1)
  #define UCKEY_ALT_F8    (-1)
  #define UCKEY_ALT_F9    (-1)
  #define UCKEY_ALT_F10   (-1)

  #define UCKEY_PREFIX    1024
  #define UCKEY_ALT_1     (UCKEY_PREFIX + '1')
  #define UCKEY_ALT_2     (UCKEY_PREFIX + '2')
  #define UCKEY_ALT_3     (UCKEY_PREFIX + '3')
  #define UCKEY_ALT_4     (UCKEY_PREFIX + '4')
  #define UCKEY_ALT_5     (UCKEY_PREFIX + '5')
  #define UCKEY_ALT_6     (UCKEY_PREFIX + '6')
  #define UCKEY_ALT_7     (UCKEY_PREFIX + '7')
  #define UCKEY_ALT_8     (UCKEY_PREFIX + '8')
  #define UCKEY_ALT_9     (UCKEY_PREFIX + '9')
  #define UCKEY_ALT_0     (UCKEY_PREFIX + '0')
  #define UCKEY_ALT_MINUS (UCKEY_PREFIX + '-')
  #define UCKEY_ALT_EQ    (UCKEY_PREFIX + '=')

  #define UCKEY_ALT_Q     (UCKEY_PREFIX + 'q')
  #define UCKEY_ALT_W     (UCKEY_PREFIX + 'w')
  #define UCKEY_ALT_E     (UCKEY_PREFIX + 'e')
  #define UCKEY_ALT_R     (UCKEY_PREFIX + 'r')
  #define UCKEY_ALT_T     (UCKEY_PREFIX + 't')
  #define UCKEY_ALT_Y     (UCKEY_PREFIX + 'y')
  #define UCKEY_ALT_U     (UCKEY_PREFIX + 'u')
  #define UCKEY_ALT_I     (UCKEY_PREFIX + 'i')
  #define UCKEY_ALT_O     (UCKEY_PREFIX + 'o')
  #define UCKEY_ALT_P     (UCKEY_PREFIX + 'p')

  #define UCKEY_ALT_A     (UCKEY_PREFIX + 'a')
  #define UCKEY_ALT_S     (UCKEY_PREFIX + 's')
  #define UCKEY_ALT_D     (UCKEY_PREFIX + 'd')
  #define UCKEY_ALT_F     (UCKEY_PREFIX + 'f')
  #define UCKEY_ALT_G     (UCKEY_PREFIX + 'g')
  #define UCKEY_ALT_H     (UCKEY_PREFIX + 'h')
  #define UCKEY_ALT_J     (UCKEY_PREFIX + 'j')
  #define UCKEY_ALT_K     (UCKEY_PREFIX + 'k')
  #define UCKEY_ALT_L     (UCKEY_PREFIX + 'l')

  #define UCKEY_ALT_Z     (UCKEY_PREFIX + 'z')
  #define UCKEY_ALT_X     (UCKEY_PREFIX + 'x')
  #define UCKEY_ALT_C     (UCKEY_PREFIX + 'c')
  #define UCKEY_ALT_V     (UCKEY_PREFIX + 'v')
  #define UCKEY_ALT_B     (UCKEY_PREFIX + 'b')
  #define UCKEY_ALT_N     (UCKEY_PREFIX + 'n')
  #define UCKEY_ALT_M     (UCKEY_PREFIX + 'm')

  #define UCKEY_PPAGE KEY_PPAGE
  #define UCKEY_NPAGE KEY_NPAGE
  #define UCKEY_UP    KEY_UP
  #define UCKEY_DOWN  KEY_DOWN
  #define UCKEY_LEFT  KEY_LEFT
  #define UCKEY_RIGHT KEY_RIGHT
  #define UCKEY_END   KEY_END
  #define UCKEY_HOME  KEY_HOME
  

#endif

///// common again /////

  #define UCKEY_INSERT    KEY_IC
  #define UCKEY_INS       KEY_IC
  #define UCKEY_DELETE    KEY_DC
  #define UCKEY_DEL       KEY_DC

  bool MonochromeConsole();

  int ConInit();
  void ConDone();

  void ConSuspend(); // suspends console (before system() for example)
  void ConRestore(); // restores console after suspend

  void ConCE( int attr = -1 ); // clear to end-of-line
  void ConCS( int attr = -1 ); // clear screen

  void ConOut( int x, int y, const char *s );
  void ConOut( int x, int y, const char *s, int attr );
  void ConPuts( const char *s );
  void ConPuts( const char *s, int attr );

  char ConGetScr(int x, int y);

  void ConCHide(); // cursor hide
  void ConCShow(); // cursor show

  void ConRefresh();

  int ConMaxX();
  int ConMaxY();
  int ConX();
  int ConY();

  void ConFG( int color );
  void ConBG( int color );
  void ConTA( int attr );

  void ConXY( int x, int y ); // go to x,y

  int ConKbHit();
  int ConGetch();

  void ConBeep();

#endif //_UNICON_H_


